/*
	File:		FileTest.c

	Contains:	Shell for folio call test tools

	Written by:	Cyrus Hazari

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Inherited 	4/25/94		
	from 		Eric Hegstrom

	Change History (most recent first):

		<4>	5/2/94		CH		Load executable code modules from cd-rom image to test "laytool" and "ls"
		<3>	4/26/94		CH		Took out Eric's MountMacFileSystem stuff at his request - they don't work
		<2>	4/25/94		CH		Added test calls for ExecuteAsSubroutine, ExecuteAsThread, LoadCode & UnloadCode for 1.3a4
		<1>	12/1/93		MT		Version 1.0 of FileTest.c.
								
	To Do:
		MT	1)	Define parameters for directory depth testing.  See OpenDirPath.c
*/


/***** Checklist:

	 Change the "File" line above to reflect the name of your tool
	 Change the "Contains" line above to reflect the call category and folio being tested
	 Change the name in the "Written by" line above to your name
	 Add Change History lines above the previous lines:
		- Thereafter, increment the change number, specify the date, your initials, and a brief description of the change(s)
		- When you first change this file, replace with "<1> (date) (initials) Derived from shell (version)"
	 Look for bulleted () HINTS below.
*/

/* HINTS:

	 If you don't know how to do something, try to write the syntax of the routine, and use the word
		UNIMPLEMENTED in the header so you can find it later.
*/

/***** Tool description for Intro, Exit, Help and Session Options displays *****/
/* HINTS:
	 Change as appropriate for your tool.
*/
#define		kToolDescription		"File Test"

/***** Version string *****/
/* HINTS:
		
	 Increment the main part when you rewrite the tool
	 Increment the "fractional" part when you make any other changes
*/
#define		kVersionStr		"V1.0"

/***** Programmer credit for Help and Session Options displays *****/
#define		kProgrammerName		"Cyrus Hazari"

/***** Call selection constants *****/
#define		kOpenDiskFile			1
#define		kCloseDiskFile			kOpenDiskFile
#define		kOpenDiskFileInDir		3
#define		kCreateAlias			4
#define		kReadDiskStream			5
#define		kSeekDiskStream			6
#define		kCreateDeleteFile		7
#define		kLoadProgram			8
#define		kLoadProgramPrio		9
//#define	kMountMacFileSystem		10			/* <3> */
#define		kExecuteAsSubroutine	11			/* <2> */
#define		kExecuteAsThread		12			/* <2> */
#define		kLoadUnloadCode			13			/* <2> */
#define		kExecuteSubFromCD		14			/* <4> */

/***** Default log filename: *****/
/* HINTS:
	
		 Change to (toolname).out.
*/
#define		kDefaultLogFile			"FileTest.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/* HINTS:

	 see usage of gIncrement1 in TestCallA()
	 Change the units to be meaningful for the tool, e.g., for a memory tool, kIncrementUnits = "bytes"
	 Change the default value to the minimum number of units for which the test is meaningful
*/
#define		kIncrement1Units	"units"
#define		kDefaultIncrement1	1

/***** Other test options *****/
/* HINTS:

	 Add one constant for each permissible value of each additional tool option.
	 Add a constant for the default value.
*/


/***** Include files *****/
/* HINTS:

	 Add a line for each utilities and call test header used
*/

#include "baseUtils.h"
#include "FileTest.h"

/* A useful macro */
#define	NEED_TO_TEST( kCall )		( ( gCallSelector == kFullTest ) || ( gCallSelector == kCall ) )

/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	// forced TRUE for now

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
long gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Duration */
Boolean gUseDuration;
long gDuration;

/* Automatic/Interactive */
Boolean gAutomatic;

/* Other test options */

/***** Utilities *****/
/* HINTS:

	 Anything you write that looks like it might be a generally useful call belongs here.
	 These will gradually migrate to our utilities libraries.
*/

/**** (End Utilities) ****/


#if 0
void	ShowIterationHelp( void )
/* HINTS:

	 Customize for clarity if necessary
*/
{
	kprintf("-icmin <<value>>\tIterate calls starting at <<value>>.\n");
	kprintf("-icmax <<value>>\tEnd call iterations at <<value>>.\n");
	kprintf("-itmin <<value>>\tIterate tests starting at <<value>>.\n");
	kprintf("-itmax <<value>>\tEnd test iterations at <<value>>.\n");
	kprintf( "\n");

	kprintf ( "-inc1 <<value>>\tUse <<value>> as increment.\n"); 
	kprintf( "\n");
}
#endif

void	ShowNonStdHelp( void )
/* HINTS:

	 Customize for your category's non-standard tool options
*/
{

}

void ShowSelHelp( void )
{
	kprintf("     OpenDirectorPath\t\tTest OpenDirectorPath.\n");
	kprintf("     CreateAlias\t\tTest CreateAlias.\n");
	kprintf("     OpenDiskFile\t\tTest OpenDiskFile.\n");
	kprintf("     OpenDiskFileInDir\tTest OpenDiskFileInDir.\n");
	kprintf("     CloseDiskFile\tTest CloseDiskFile.\n");
	kprintf("     ReadDiskStream\tTest ReadDiskStream.\n");
	kprintf("     SeekDiskStream\tTest SeekDiskStream.\n");
	kprintf("     CreateDeleteFile\tTest CreateFile and DeleteFile.\n");
	kprintf("     LoadProgram\tTest LoadProgram.\n");
	kprintf("     LoadProgramPrio\tTest LoadProgramPrio.\n");
	kprintf("     MountMacFileSystem\tTest MountMacFileSystem.\n");
	kprintf("     ExecuteAsSubroutine\tTest ExecuteAsSubroutine.\n");					/* <2> */
	kprintf("     ExecuteAsThread\tTest ExecuteAsThread.\n");							/* <2> */
	kprintf("     LoadUnloadCode\tTest LoadCode and UnloadCode.\n");					/* <2> */
}

void	ShowHelp( void )
/* Show Help for this tool */
{
	kprintf ( "\n#################################################################\n");
	kprintf ( "%s %s Help\n", kToolDescription, kVersionStr );
	kprintf ( "Written by %s for The 3DO Company\n", kProgrammerName );
	kprintf (    "#################################################################\n");
	kprintf( "\n");
	
	kprintf( "-out <<filename>>     \tUse specified log file.\n");
	kprintf( "                      \tDefault filename is %s.\n", kDefaultLogFile);
	kprintf( "   If not specified, no log file will be used.\n");
	kprintf( "\n");
	
	#if 0
	
	ShowIterationHelp();
	
	kprintf ( "-duration <<value>>  \tStop test iterations after <<value>> seconds.\n"); 
	kprintf( "                      \tDefault is %i seconds.\n", kDefaultDuration);
	kprintf( "   If not specified, no time limit will be used.\n");
	kprintf( "\n");
	
	#endif
	
	ShowNonStdHelp();
	
	kprintf( "-error none\tDon't check for errors.\n");
	kprintf( "       cont\tWhen an error occurs, report it and continue.\n");
	kprintf( "       stop\tWhen an error occurs, report it and stop the tool.\n");
	kprintf( "\n");

	kprintf( "-ierror none\tDon't check for incidental errors.\n");
	kprintf( "        cont\tWhen an incidental error occurs, report it and continue.\n");
	kprintf( "        stop\tWhen an incidental error occurs, report it and stop the tool.\n");
	kprintf( "\n");

	kprintf( "-time off\tDon't output timing information.\n");
	kprintf( "      on \tOutput timing information.\n");
	kprintf( "\n");

	kprintf("-func none   \tDon't verify functionality.\n");
	kprintf("      silent \tVerify functionality silently.\n");
	kprintf("      report \tVerify functionality and report useful information.\n");
	kprintf( "\n");

	kprintf("-fault off\tMake known assumptions.\n");
	kprintf("       on \tSkip known assumptions.\n");
	kprintf( "\n");

	kprintf("-sel <<no parameter>>\tRun the entire test suite.\n");
	ShowSelHelp();
	kprintf( "\n");
	
	kprintf ( "#################################################################\n");
}

void ShowIterationOptions( void )
/* HINTS:

	 Customize for clarity if necessary
*/
{
	kprintf( "\n");
	kprintf ( "Iterate each test from %i %s",gMinOuterIterations, kIncrement1Units);
	kprintf ( " to %i %s,", gMaxOuterIterations, kIncrement1Units);
	kprintf ( " in increments of %i %s.\n",gIncrement1,kIncrement1Units); 
	kprintf ( "Within each test, iterate the call from %i to %i.\n",gMinInnerIterations,gMaxInnerIterations);
}

void ShowNonStdOptions( void )
/* HINTS:

	 Customize for your tools non-standard options
		- fully describe each option's usage, as in ShowOptions() 
*/
{
}
				
void ShowSelOptions()
/* HINTS:

	 Customize for your category's calls
*/
{
	switch ( gCallSelector )
	{
		case kOpenDiskFileInDir:	/*	kOpenDiskFileInDir and kCloseDirectory are one and the same	*/
			kprintf("Test OpenDiskFileInDir and CloseDirectory.\n");
			break;

		case kCreateAlias:
			kprintf("Test Create Alias.\n");
			break;

		case kCloseDiskFile:
			kprintf("Test OpenDiskFile and CloseDiskFile.\n");
			break;
			
		case kReadDiskStream:
			kprintf("Test ReadDiskStream.\n");
			break;
			
		case kSeekDiskStream:
			kprintf("Test SeekDiskStream.\n");
			break;
			
		case kCreateDeleteFile:
			kprintf("Test CreateDeleteFile.\n");
			break;
			
		case kLoadProgram:
			kprintf("Test LoadProgram.\n");
			break;
			
		case kLoadProgramPrio:
			kprintf("Test LoadProgramPrio.\n");
			break;
			
//		case kMountMacFileSystem:															/* <3> */
//			kprintf("Test MountMacFileSystem.\n");
//			break;
			
		case kExecuteAsSubroutine:															/* <2> */
			kprintf("Test ExecuteAsSubroutine.\n");
			break;
			
		case kExecuteAsThread:																/* <2> */
			kprintf("Test ExecuteAsThread.\n");
			break;
			
		case kLoadUnloadCode:																/* <2> */
			kprintf("Test LoadCode and UnloadCode.\n");
			break;
			
		case kExecuteSubFromCD:																/* <4> */
			kprintf("Test CD-ROM image to make sure LAYTOOL works.\n");
			break;
			
	}
}
				
void ShowOptions()
/* Show the option settings for this session */
{
	kprintf ( "\n#################################################################\n");
	kprintf ( "%s %s - Session options\n", kToolDescription, kVersionStr);
	kprintf ( "Written by %s for The 3DO Company\n", kProgrammerName );
	kprintf (   "#################################################################\n");
	
	if (gUseLogFile) kprintf( "-out: Use log file: %s.\n",gLogFile); else kprintf("Do not use a log file.\n");
	
	ShowIterationOptions();

	if ( gUseDuration )
		kprintf ( "Stop test iterations after %i seconds.\n", gDuration);
	else
		kprintf ( "Don't use a time limit.\n" );
	kprintf( "\n");
	
	ShowNonStdOptions();
	
	kprintf("-auto: ");
	if ( gAutomatic ) kprintf("Run non-interactively.\n"); else kprintf("Run with joypad interaction.\n");

	kprintf( "-error: ");
	switch (gCheckErrors)
	{
		case kNone:
			kprintf("Don't check for errors.\n");
			break;
			
		case kCont:
			kprintf("When an error occurs, report it and continue.\n");
			break;
			
		case kStop:
			kprintf("Stop the tool on the first error to occur.\n");
			break;
		
	} // end switch

	kprintf( "-ierror: ");
	switch (gCheckIncidentalErrors)
	{
		case kNone:
			kprintf("Don't check for incidental errors.\n");
			break;
			
		case kCont:
			kprintf("When an incidental error occurs, report it and continue.\n");
			break;
			
		case kStop:
			kprintf("Stop the tool on the first incidental error to occur.\n");
			break;
		
	} // end switch

	kprintf("-time: ");
	if (gOutputTiming) kprintf("Output timing information.\n"); else kprintf("Don't output timing information.\n");

	kprintf("-func: ");
	switch (gVerify)
	{
		case kNoVerify:
			kprintf("Don't verify functionality.\n");
			break;
			
		case kSilent:
			kprintf("Verify functionality silently.\n");
			break;
			
		case kReport:
			kprintf("Verify functionality and report useful information.\n");
			break;
		
		} // end switch

	kprintf("-fault: ");
	if (gCheckFault) kprintf("Skip known assumptions.\n"); else kprintf("Make known assumptions.\n");

	kprintf("-sel: ");
	switch (gCallSelector)
	{
		case kFullTest:
			kprintf("Run the entire test suite.\n");
			break;
			
		default:
			ShowSelOptions();
			break;
	} // end switch
	
	kprintf ( "#################################################################\n");
	
}

void GetNonStdParams(int theArgCount, char** theArgList, int *paramIndx)
/* Get non-standard tool options from the command line arguments */
{

}

void InitNonStdParams( void )
/* HINTS:

	 Initialize the globals for each additional tool option
*/
{

}

long GetCallSelector( char *aStr )
/* HINTS:

	 Customize for your call names and constants
*/
{
	if (strcasecmp(aStr, "OpenDiskFile") == 0)
		return kOpenDiskFile;
	
	if (strcasecmp(aStr, "OpenDiskFileInDir") == 0)
		return kOpenDiskFileInDir;
	
	if (strcasecmp(aStr, "CloseDiskFile") == 0)
		return kCloseDiskFile;

	if (strcasecmp(aStr, "CreateAlias") == 0)
		return kCreateAlias;
		
	if (strcasecmp(aStr, "ReadDiskStream") == 0)
		return kReadDiskStream;
	
	if (strcasecmp(aStr, "SeekDiskStream") == 0)
		return kSeekDiskStream;

	if (strcasecmp(aStr, "CreateDeleteFile") == 0)
		return kCreateDeleteFile;
		
	if (strcasecmp(aStr, "LoadProgram") == 0)
		return kLoadProgram;
		
	if (strcasecmp(aStr, "LoadProgramPrio") == 0)
		return kLoadProgramPrio;
		
//	if (strcasecmp(aStr, "MountMacFileSystem") == 0)									/* <3> */
//		return kMountMacFileSystem;
		
	if (strcasecmp(aStr, "ExecuteAsSubroutine") == 0)									/* <2> */
		return kExecuteAsSubroutine;
		
	if (strcasecmp(aStr, "ExecuteAsThread") == 0)										/* <2> */
		return kExecuteAsThread;
		
	if (strcasecmp(aStr, "LoadUnloadCode") == 0)										/* <2> */
		return kLoadUnloadCode;
		
	if (strcasecmp(aStr, "ExecuteSubFromCD") == 0)										/* <4> */
		return kExecuteSubFromCD;
		
	return -1;
}

Boolean	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	int			paramIndx = 0;
	Boolean		done = false;
	char		*tempStr;
	long		aCallSelector;
	

	gShowStatus = TRUE;
	gUseDuration = FALSE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

	
/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kNoVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;
	gDuration = kDefaultDuration;
	gAutomatic = kDefaultAutomatic;

	
/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	InitNonStdParams();

	
	while ( !done )
	{
	
		if (strcasecmp(theArgList[paramIndx], "?") == 0)
		{
			ShowHelp();
			return kShowHelp;
		}
		else if (strcasecmp(theArgList[paramIndx], "-out") == 0)
			{
				/* Need to use a log file:
					 If the user specifies a name, use it.
					 Otherwise, the default will be used. */

				if ( GetNextParam( theArgCount, theArgList, paramIndx, &tempStr) )
				{
					strcpy(gLogFile,tempStr);
					paramIndx++;
				}
				
				gUseLogFile = OpenMacLink();
				if (!gUseLogFile)
					kprintf("Can't open the log file");
			}
			
		else if ( GetNextNumParam( theArgCount, theArgList, &paramIndx, "-icmin", &gMinInnerIterations) )
			{
			}
						
		else if ( GetNextNumParam( theArgCount, theArgList, &paramIndx, "-icmax", &gMaxInnerIterations) )
			{
			}
						
		else if ( GetNextNumParam( theArgCount, theArgList, &paramIndx, "-itmin", &gMinOuterIterations) )
			{
			}
						
		else if ( GetNextNumParam( theArgCount, theArgList, &paramIndx, "-itmax", &gMaxOuterIterations) )
			{
			}
						
		else if (strcasecmp(theArgList[paramIndx], "-error") == 0)
			{
				if ( GetNextParam( theArgCount, theArgList, paramIndx, &tempStr) )
				{
					if ( strcasecmp(tempStr, "none") == 0 )
						{
						gCheckErrors = kNone;
						paramIndx++;
						}
					else if (strcasecmp(tempStr, "cont") == 0)
						{
						gCheckErrors = kCont;
						paramIndx++;
						}
					else if (strcasecmp(tempStr, "stop") == 0)
						{
						gCheckErrors = kStop;
						paramIndx++;
						}
					else if (strlen(tempStr) != 0) gInvalidArg = true;
				}
			}

		else if (strcasecmp(theArgList[paramIndx], "-ierror") == 0)
			{
				if ( GetNextParam( theArgCount, theArgList, paramIndx, &tempStr) )
				{
					if ( strcasecmp(tempStr, "none") == 0 )
						{
						gCheckIncidentalErrors = kNone;
						paramIndx++;
						}
					else if (strcasecmp(tempStr, "cont") == 0)
						{
						gCheckIncidentalErrors = kCont;
						paramIndx++;
						}
					else if (strcasecmp(tempStr, "stop") == 0)
						{
						gCheckIncidentalErrors = kStop;
						paramIndx++;
						}
					else if (strlen(tempStr) != 0) gInvalidArg = true;
				}
			}

		else if (strcasecmp(theArgList[paramIndx], "-func") == 0)
			{
				if ( GetNextParam( theArgCount, theArgList, paramIndx, &tempStr) )
				{
					if ( strcasecmp(tempStr, "none") == 0 )
					{
						gVerify = kNoVerify;
						paramIndx++;
					}
					else if (strcasecmp(tempStr, "silent") == 0)
					{
						gVerify = kSilent;
						paramIndx++;
					}
					else if (strcasecmp(tempStr, "stop") == 0)
					{
						gVerify = kReport;
						paramIndx++;
					}
					else if (strlen(tempStr) != 0) gInvalidArg = true;
				}
			}

		else if (GetNextFlagParam(theArgCount, theArgList, &paramIndx, "-time", &gOutputTiming))
			{
			}
			
		else if (GetNextFlagParam(theArgCount, theArgList, &paramIndx, "-fault", &gCheckFault))
			{
			}
			
		else if (strcasecmp(theArgList[paramIndx], "-sel") == 0)
			{
				if ( GetNextParam( theArgCount, theArgList, paramIndx, &tempStr) )
				{
					if ( ( aCallSelector = GetCallSelector( tempStr ) ) > 0 )
					{
						gCallSelector = aCallSelector;
						paramIndx++;
					}
					else
						if ( strlen(tempStr) != 0 ) gInvalidArg = true;
				}
			}
			
		else if ( GetNextNumParam( theArgCount, theArgList, &paramIndx, "-inc1", &gIncrement1) )
			{
			}
						
		else if ( GetNextNumParam( theArgCount, theArgList, &paramIndx, "-duration", &gDuration) )
			{
				gUseDuration = TRUE;
			}
						
		else if (GetNextFlagParam(theArgCount, theArgList, &paramIndx, "-auto", &gAutomatic))
			{
			}
		
		else GetNonStdParams(theArgCount, theArgList, &paramIndx);
						
		if ( gInvalidArg )
		{
			ShowHelp();
			return kAbortTest;
		}
			
		paramIndx++;
//		if ( paramIndx > theArgCount )
		if ( paramIndx >= theArgCount )
			done = true;
		
	}	//	end while
	

	
	return kRunTest;
}	// end GetParams



/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory()
/* HINTS:

	 Customize for your category's calls
*/
{
	if ( NEED_TO_TEST( kOpenDiskFile ) )
		TestOpenDiskFile(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kOpenDiskFileInDir ) )
		TestOpenDiskFileInDir(gMinOuterIterations, gMaxOuterIterations);
		
	if ( NEED_TO_TEST( kCreateAlias ) )
		TestCreateAlias(gMinOuterIterations, gMaxOuterIterations);
		
	if ( NEED_TO_TEST( kReadDiskStream ) )
		TestReadDiskStream(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kSeekDiskStream ) )
		TestSeekDiskStream(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kCreateDeleteFile ) )
		TestCreateDeleteFile(gMinOuterIterations, gMaxOuterIterations);
		
	if ( NEED_TO_TEST( kLoadProgram ) )
		TestLoadProgram(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kLoadProgramPrio ) )
		TestLoadProgramPrio(gMinOuterIterations, gMaxOuterIterations);

//	if ( NEED_TO_TEST( kMountMacFileSystem ) )											/* <3> */
//		TestMountMacFileSystem(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kLoadUnloadCode ) )												/* <2> */
		TestLoadUnloadCode(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kExecuteAsSubroutine ) )											/* <2> */
		TestExecuteAsSubroutine(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kExecuteAsThread ) )												/* <2> */
		TestExecuteAsThread(gMinOuterIterations, gMaxOuterIterations);

	if ( NEED_TO_TEST( kExecuteSubFromCD ) )											/* <4> */
		TestExecuteSubFromCD(gMinOuterIterations, gMaxOuterIterations);

}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram()
{
	int32 theError;
	
	gThisTask = GetMyItem();

	if ( OpenAudioFolio() )
	{
		kprintf("Can't open the audio folio!");
		return FALSE;
	}
	
	theError = SleepSeconds( 2 );
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
#if 0	
	if (!SetDirectory())
	{
		kprintf("Unable to set Working Directory!!\n");
		return FALSE;
	}
	if (! MakeAliases())
	{
		kprintf("Unable to make Aliases!!\n");
		return FALSE;
	}
#else

	MakeAliases();

#endif

	return TRUE;
}


int main (int argc, char **argv)
{
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{
#include "f1.h"

			kprintf("After GetParams()\n\n");
			ShowOptions();
			kprintf("After ShowOptions()\n\n");
	
			if (gUseLogFile) 
			{
				gTheFile  = fopen(gLogFile,"w");
				pipeData("## Begin %s\n", kToolDescription);
				pipeData("##\n");
			}
			
			TestCategory(); 
			
			if (gUseLogFile)  
			{
				pipeData("##\n");
				pipeData("## End %s\n", kToolDescription);
				fclose(gTheFile);
			}
				
		}

		kprintf ( "#################################################################\n");
		kprintf ( "%s - End\n", kToolDescription );
		kprintf ( "#################################################################\n");


	}
	
 }






