/*
	File:		OpenDirPath.c

	Contains:	

	Written by:	

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 12/01/93	MT		Derived from TestCallA.c

	To Do:
	
		1)	an error is set whenever OpenDirectoryPath returns NULL.
			Need to check if the condition is actually an error.  Specifically,
			  Check if attempted to open a file
			  Check if attempted to open a nonexistent directory.
		2)	Allow user to specify a test of a depth of directory.  Don't know if
			this is possible or even feasible.
		3)	Define a verification routine
		4)	How do you check results of CloseDirectory or OpenDirectoryPath?
			
*/


/***** Checklist:

	 Change the "File" line above to reflect the name of your call
	 Change the "Contains" line above to reflect the call being tested
	 Change the name in the "Written by" line above to your name
	 Add Change History lines above the previous lines:
		- Increment the change number
		- Specify the date, your initials, and a brief description of the change(s)
		- When you first change this file, add "<2> (date) (initials) Derived from shell"
*/

/* HINTS:

	 Change to reflect your call name.
*/
#include "DirTest.h"
#include "directory.h"
#include "directoryfunctions.h"
#include "data.h"
#include "Configuration.h"

/* Actual call name */
#define	kCallNameStr	"OpenDirectoryPath"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s:\n"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i\n"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i\n"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr		"%s (%i)\n"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i\n:  PATH:  %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s\n"

/**** Test routine macros ****/
/* HINTS:

	 See TestCallA below for usage.
*/
#define		PIPE_TIMING_HEADER		pipeData(kTimingHeaderStr,callName)
#define		PIPE_OUTER_TIMING		pipeData(kOuterTimingStr,minOuterIterations,lastIteration, accumulator)
#define		INNER_ITERATIONS		OpenDirPath(indx, gMinInnerIterations, gMaxInnerIterations)
/* HINTS:

	 Change to use your call name and add the call result and parameters to the parameter list.
*/
	
extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern long gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Duration */
extern long gDuration;
extern Boolean gUseDuration;

/* Automatic/interactive */
extern Boolean gAutomatic;

extern	char *gpathName;
extern	int	changeDir(char *);

/***** Test routines *****/
Boolean		VerifyOpenDirPath ( Boolean beforeCall, void *data )
/* Functionality verification for OpenDirectoryPath */
/* HINTS:

	 In all likelihood, parameter list and verification format will change drastically depending on the call
	 It may be desirable to use static variables for pre- and post- call comparison
*/
{
	static char	path[256] = "";
	Directory	*DirP;
	static Item	retItem;
	int32		i;
	ItemNodeP	ptr;
	Boolean		result;
	
	if (beforeCall == kBeforeCall)
	{	
		memcpy(path, data, strlen((char *)data)+1);
		if (!strcmp(path,".")
			|| !strcmp(path, ""))
			GetDirectory(path, 256);
		else
			if (!strcmp(path, ".."))
			{
				GetDirectory(path, 256);
				i = strlen(path);
				while (i > 0 && (path[i] != '/'))
					i--;
				if (path[i] == '/')
					path[i] = 0;
			}

		return true;
	}


	DirP = (Directory *)data;
	retItem = DirP->dir_OpenFileItem;

	if (retItem >= 0)
	{
		ptr = (ItemNodeP)LookupItem(retItem);
		
		pipeData("%s\n",ptr == NULL ? "NON-EXISTENT": "EXISTS" );
					
		if (ptr == NULL)
			return FALSE;

		i = strlen(path);
		while ((i > 0) && (path[i] != '/'))
			i--;
		
		i += path[i] == '/';
		result = !strcmp(path+i, ptr->n_Name)	&& (ptr->n_SubsysType == FILESYSTEMNODE);

		return result;
	}
	return	FALSE;
	
}

long	OpenDirPath (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	int32	dirIndx = 0;
	Directory	*dirP = NULL;
	

	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (theError)
		{
			pipeData(kIncidentalErrorStr, kCallNameStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		for ( dirIndx = 0; dirIndx < NUM_CASES; dirIndx++ )
		{
//			Item	dirItem;

			if (gVerify != kNoVerify)
				VerifyOpenDirPath(kBeforeCall, dirs[dirIndx]);
				
			startTime = TICKCOUNT(); 		
			dirP = OpenDirectoryPath( dirs[ dirIndx ] );

			endTime = TICKCOUNT();
			accumulator += endTime - startTime;

			if ( gVerify != kNoVerify )
			{
				theError = VerifyOpenDirPath(kAfterCall, dirP);
				if (theError)
				{
					pipeData(kCallErrorStr, indx, dirs[dirIndx]);
					if (gCheckErrors == kStop)	return kBreakTime;
				}
			}
#ifdef CHECK_CloseDirectory_PARAM
			if( dirP !=0 )				/* check for null pointer - is'nt it better for the call return an error if invalid pointer is passed?*/
			{
				CloseDirectory( dirP );
			}
#else
			CloseDirectory( dirP );		/*	DOES NOT CHECK FOR dirP's contents	*/
#endif
		}
#if 0
		/* Functionality verification (post-call) */
		if ( gVerify != kNoVerify ) theError = !VerifyCallA ( kAfterCall, indx );
#endif
		/* HINTS:
		
			 Call name, return value, and additional parameters depend on the call
				and verification requirements.
		*/
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				pipeData(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}
#if 0
	if (gOutputTiming)
		pipeData(kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
#endif	
	return accumulator;
}

void TestOpenDirPath( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
	long	startTime;
	long	elapsedTime;
	long	duration = SecondsToAudioTicks( gDuration );
	Boolean keepGoing = FALSE;
	long	lastIteration = 0;
		
	callName = kCallNameStr;
	STATUS(callName);
	if (gOutputTiming)
		PIPE_TIMING_HEADER;

	startTime = TICKCOUNT();

	do
	{
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			lastIteration = indx;
			
			theTime = INNER_ITERATIONS;

			accumulator += theTime;
			if (theTime == kBreakTime) break;
			
			if ( gUseDuration )
			{
				elapsedTime = TICKCOUNT() - startTime;
				if ( elapsedTime >= duration ) break;
			}
		}
	
		if (gOutputTiming)
			PIPE_OUTER_TIMING;
			
		if ( gUseDuration )
			keepGoing = ( elapsedTime < duration );
			
	} while ( keepGoing );
	
}


