/*** Copyright  1993 The 3DO Company ***/

/********************************************************************
* All rights reserved.
* This material constitutes confidential and proprietary information
* of the 3DO Company and shall not be used by any Person or for any
* purpose except as expressly authorized in writing by the 3DO Company.
********************************************************************/


/* Include this when using the Audio Folio */

#include "audioUtils.h"
#include "availMem.h"
#include "stdio.h"

#define MAXPOINTS  (40) 


/* Macro to simplify error checking.*/
#define CHECKRESULT(val,name) \
	if (val < 0) \
	{ \
		Result = val; \
		printf("Failure in %s: $%x\n", name, val); \
		goto cleanup; \
	}

/*  type disagreement with something in baseUtils???
int32 srand(int32 range );
int32 rand( void );
*/

void pipeErr(int32 theError) 
{	
	char errStringBuff[256];
	
	GetSysErr(&errStringBuff[0], sizeof(errStringBuff), theError);
	printf(errStringBuff);
	printf("\n");
	
}

/****************************  Choose   *******************************/
/* choose a random number between 0 and range */

int32 Choose ( int32 range )
{
	int32 val, r16;
	static Boolean firstTime = TRUE;
	
	if (firstTime) {
		srand(121);
		firstTime = FALSE;
	}

	r16 = rand() & 0xFFFF;
	val = (r16*range) >> 16;
	return val;
}


/****************************  WrapAround   *******************************/
int32 WrapAround(int32 val, int32 min, int32 max)
{
	while (val > max) val = val - max + min - 1;
	while (val < min) val = val + max - min + 1;
	return val;
}

/****************************  MinOfZero   *******************************/
int32 MinOfZero(int32 x)
{
	if (x<0) return 0;
	else return x;
}

/****************************  InitAudio   *******************************/
int32 InitAudio(void)
{
	int32 Result=0;


/* Initialize audio, return if error. */ 
		
	if (OpenAudioFolio())
	{
		printf("Audio Folio could not be opened!\n");
		return(-1);
	}
	/* FindMathFolio to get MathBase */
#ifdef cardinal2b
	Result = FindMathFolio();
#else
	Result = OpenMathFolio();
#endif
	if (Result < 0)
	{
		printf("FindMathFolio() failed! Did you run operamath?\n");
		PrintfSysErr(Result);
		return Result;
	}

	return Result;
}


/*********************************************************************/
int32 SetupInstrument(Item *ourInst, char *instToLoadName, Item connectInst, char *sourceKnobOne, char *inputKnobOne)
{
int32 Result = 0;
Item ourInstItem;	
	
	*ourInst = LoadInstrument(instToLoadName, DEFAULT_OUT_DEVICE, DEFAULT_PRIORITY);
	CHECKRESULT(*ourInst,"LoadInstrument");
	ourInstItem = *ourInst;
	
	if (!(sourceKnobOne ==  (void *) NULL)) {
		Result = ConnectInstruments (ourInstItem, sourceKnobOne, connectInst, inputKnobOne);
		CHECKRESULT(Result,"ConnectInstruments");
	}
	
cleanup:
	return Result;
}

 
/****************************  StartTheSound   *******************************/
int32 StartTheSound(Item instrumentToStart, Item requestKnobToTweak, int32 atFrequency, int32 atAmplitude)
{
int32 Result = -1;
/* Define Tags for StartInstrument */
static TagArg AmpFreqTags[] =
	{
		{ AF_TAG_AMPLITUDE, 0},
		{ AF_TAG_FREQUENCY, 0},
      	  	{ TAG_END, 0 }
    	};

	AmpFreqTags[0].ta_Arg = (void *) 0;    /*  start with an amplitude of zero  */
	AmpFreqTags[1].ta_Arg = (void *) atFrequency;
	Result = StartInstrument( instrumentToStart,  &AmpFreqTags[0] );   
	CHECKRESULT(Result,"StartInstrument");
	SleepAudioTicks(RAMPDOWNTIME);   /* sleep a dash after Start command */
	
/* Go slowly to full amplitude. */
	Result = TweakRawKnob(requestKnobToTweak, atAmplitude);
	CHECKRESULT(Result,"TweakRawKnob");
	
cleanup:	
	return Result;
}


/****************************  StopTheSound   *******************************/
int32 StopTheSound (Item instrumentToStop, Item requestKnobToTweak) 
{
int32 Result = -1;

/* Go slowly to zero. */

// TO DO:  check for and return errors

	Result = TweakRawKnob(requestKnobToTweak, 0);
	CHECKRESULT(Result,"StopTheSound -- TweakRawKnob");
	
	SleepAudioTicks(RAMPDOWNTIME);
	
	Result = StopInstrument( instrumentToStop, NULL ); 	/* stop it NOW!! */
	CHECKRESULT(Result,"StopTheSound -- StopInstrument");
cleanup:
	return Result;
}


/****************************  StartPitchNote   *******************************/
int32 StartPitchNote ( Item Instrument,  Item requestKnobToTweak, int32 Note, int32 Velocity)
{
	int32 Result;
	int32 atAmplitude = 0x7FFF;
	static TagArg VelPitchTags[] =
	{
		{ AF_TAG_VELOCITY, 0},
		{ AF_TAG_PITCH, 0},
       		 { TAG_END, 0 }
   	 };


	VelPitchTags[0].ta_Arg = (void *) Velocity;
	VelPitchTags[1].ta_Arg = (void *) Note;
	Result = StartInstrument( Instrument, &VelPitchTags[0] );
	CHECKRESULT(Result,"StartInstrument");
	SleepAudioTicks(RAMPDOWNTIME);   /* sleep a dash after Start command */
	
	Result = TweakRawKnob(requestKnobToTweak, atAmplitude);
	CHECKRESULT(Result,"TweakRawKnob");
	
	return 0;
	
cleanup:	
	return Result;
}


/****************************  PlayPitchNote   *******************************/
int32 PlayPitchNote ( Item Instrument,  Item requestKnobToTweak, int32 Note, int32 Velocity, int32 Length, int32 ReleaseLength)
{
	int32 Result;

	StartPitchNote (Instrument,  requestKnobToTweak, Note, Velocity);
	CHECKRESULT(Result,"StartPitchNote");

	Result = SleepAudioTicks( Length );  						/*  length of note   */
	CHECKRESULT(Result,"SleepAudioTicks");
	
	Result = ReleaseInstrument( Instrument, NULL);
	CHECKRESULT(Result,"ReleaseInstrument");
	Result = SleepAudioTicks( ReleaseLength ); 				/*  length to wait after releasing note   */
	CHECKRESULT(Result,"SleepAudioTicks");
	Result = StopTheSound(Instrument, requestKnobToTweak);
	CHECKRESULT(Result,"StopTheSound");
		
	return 0;
	
cleanup:	
	return Result;
}

/****************************  SetAudioTagInfo   *******************************/
int32 SetAudioTagInfo(Item AnAudioItem, int32 anAFTag, int32 Value )
{
	TagArg Tags[3];
	int32 Result = 0;

/* Get data from sample by sending a Taglist that gets filled in with the  current values. */
	Tags[0].ta_Tag = anAFTag;
	Tags[0].ta_Arg = (void *)Value;
	Tags[1].ta_Tag = TAG_END;
	Result = SetAudioItemInfo(AnAudioItem, Tags);
	CHECKRESULT(Result,"GetAudioItemInfo");
		
	return 0;
	
cleanup:	
	return Result;

}

/****************************  GetAudioTagInfo   *******************************/
int32 GetAudioTagInfo(Item AnAudioItem, int32 anAFTag )
{
	TagArg Tags[3];
	int32 Data;
	int32 Result = 0;

/* Get data from sample by sending a Taglist that gets filled in with the  current values. */
	Tags[0].ta_Tag = anAFTag;
	Tags[0].ta_Arg = NULL;
	Tags[1].ta_Tag = TAG_END;
	Result = GetAudioItemInfo(AnAudioItem, Tags);
	CHECKRESULT(Result,"GetAudioItemInfo");
	
	Data = (int32) Tags[0].ta_Arg;   /* Points to requested information. */
	
	return Data;
	
cleanup:	
	return Result;

}


/****************************  ChangeKnob   *******************************/
 Item ChangeKnob(Item Instrument, char *KnobName, int32 Value, int ReleaseIt)
{
	Item newKnob;
	int32 Result = 0;

	newKnob = GrabKnob(Instrument, KnobName);
	CHECKRESULT(newKnob,"ChangeKnob -- GrabKnob");
	
	Result = TweakRawKnob(newKnob, Value);
	CHECKRESULT(Result,"ChangeKnob --GrabKnob");
	
	if (ReleaseIt) {
		Result = ReleaseKnob(newKnob);
		CHECKRESULT(Result,"ChangeKnob --ReleaseKnob");
		return 0;
	}
	
	return newKnob;
	
cleanup:	
	return Result;
}

/********************************* CreateAttachment *********************************/
Item CreateAttachment( Item Instrument, Item Envelope, char *FIFOName, uint32 Flags)
{
/* Declare local variables */
	Item 	Result;
	TagArg 	Tags[5];

/* Initalize local variables */
	Result = -1;

	Tags[0].ta_Tag = AF_TAG_HOOKNAME;
	Tags[0].ta_Arg = (void *) FIFOName;
	Tags[1].ta_Tag = AF_TAG_ENVELOPE;
	Tags[1].ta_Arg = (void *) Envelope;
	Tags[2].ta_Tag = AF_TAG_INSTRUMENT;
	Tags[2].ta_Arg = (void *) Instrument;
	Tags[3].ta_Tag = AF_TAG_SET_FLAGS;
	Tags[3].ta_Arg = (void *) Flags;
	Tags[4].ta_Tag =  TAG_END;

    	Result = CreateItem( MKNODEID(AUDIONODE,AUDIO_ATTACHMENT_NODE), Tags );
	CHECKRESULT(Result,"CreateItem");

	return Result;
	
cleanup:	
	return Result;
}

#define MAXTUNINGFREQS (36)

Item DoTuning(Item InsToTune, int32 BaseFreq, int32 BaseNote, int32 NotesPerOctave, int32 *Cents)
/*  
	BaseFreq is in hertz, 
	BaseNote is a midi Note number
	NotesPerOctave is obvious
	Cents is a pointer to an array of cents above is octave's base pitch (1200 cents per octave)
		equal-tempered= {0,100,200,300,400,500,600,700,800,900,1000,1100,1200}
		
	Translation from Cents to freqs:
		convert to ratio, then add the
		ThisFreq = ((Cents / 1200) * ThisOctaveBaseFreq ) + ThisOctaveBaseFreq;
				
*/
{
	int32 		Result = 0, Index;
	ufrac16	TuningTable[MAXTUNINGFREQS];
	frac16 		FrBaseFreq;
	Item		ourTuning;
	
/* Calculate frequencies as ratios of cents from the base frequency. */
	FrBaseFreq = (ufrac16) Convert32_F16(BaseFreq);
	
	TuningTable[0] = FrBaseFreq;  
	/*PRT(("Variable: %s, Value:  ", "TuningTable[0]"));
	PrintFrac16( (frac16) TuningTable[0] );
	kprintf("\n");*/

	for (Index =1; (Index < (NotesPerOctave+1)); Index++){
		TuningTable[Index] = (ufrac16) AddF16(MulUF16(FrBaseFreq, DivUF16(Cents[Index], 1200)), FrBaseFreq);
		/*PRT(("Variable: %s, Value:  ", "TuningTable[Index]"));
		   PrintFrac16( (frac16) TuningTable[Index] );
		kprintf("\n");*/

	}
		
/* Create a tuning item that can be used with many instruments. */
	ourTuning = CreateTuning( TuningTable, NotesPerOctave, NotesPerOctave, BaseNote );
	CHECKRESULT(ourTuning,"CreateTuning");
	
/* Tell an instrument to use this tuning. */
	Result = TuneInstrument( InsToTune, ourTuning );
	CHECKRESULT(Result,"TuneInstrument");
	
	return ourTuning;
	
cleanup:	
	return Result;	
}



void PrintFrac16 (frac16 num)
{
	int32 integer;
	uint32 temp;
	int i;
	
	integer=num>>16;
	temp=(num&0xFFFF);
	printf("%d.",integer);
	for (i=1;i<=6;i++)
	{
		temp=MulUF16(temp,Convert32_F16(10));
		printf("%d",((temp&0xF0000)>>16));
		temp=(temp&0xFFFF);
	}
}


/*GLOBAL  definition of Mem test*/
int gMemTest = 1;  //HAAAAAAAAAACCKKKKKKKKKKK HACK

int32 doMemTest( Boolean before)
{
	static uint32 beforeAmount;
	uint32 afterAmount;

	if (before == kBeforeCall) {		
		if ( gMemTest )  beforeAmount = TotalAvailTaskMem() + TotalAvailKernalMem();
	}
	else {
		if ( gMemTest ) {
			afterAmount = TotalAvailTaskMem() + TotalAvailKernalMem();
			if (beforeAmount != afterAmount) {
				printf("mem before: 0x%x, after: 0x%x\n", beforeAmount, afterAmount);
				return false;
			}
		}
	}
	return true;
}
