/*
 * Bounce - Program to read Analog data from the Analog streamer,
 * and bounce the picture around the screen.
 */

#undef ORIGINAL
#undef CYRUS_1
//#define CYRUS_1 1
#define ROD_1 1
//#undef ROD_1
#undef FLIP_ROTATE
//#define FLIP_ROTATE 1

#define PHASE_IN_SPEED	10
#define PHASE_OUT_SPEED	10

#include "types.h"
#include "mem.h"
#include "graphics.h"
#include "av.h"
#include "JoyPad.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "ctype.h"


#include "Portfolio.h"
#include "Utils3DO.h"

#define REPORT_ERR(x) {if ((x) < 0) PrintfSysErr(x);}
#define SC_WIDTH 320
#define SC_HEIGHT 240
#define IMAGE_WIDTH (SC_WIDTH / 2)
#define IMAGE_HEIGHT (SC_HEIGHT / 2)
#define BUFFERS 4
#define CAPTUREBUFS (BUFFERS / 2)
#define VIDEOBUFS (BUFFERS / 2)
#define BUFSIZE (SC_WIDTH * SC_HEIGHT * 2)
#define BOUNCESPEED 8
#define BOUNCESPEEDMASK (BOUNCESPEED - 1)

typedef struct Global
{
	Item streamer;
	Item stream;
	ubyte *capture[CAPTUREBUFS];
	ubyte *video[VIDEOBUFS];
	Item cBitmapsI[CAPTUREBUFS];
	Bitmap *cBitmaps[CAPTUREBUFS];
	Item screensI[VIDEOBUFS];
	Screen *screens[VIDEOBUFS];
	Item screenGroup;
	Item vBitmapsI[VIDEOBUFS];
	Bitmap *vBitmaps[VIDEOBUFS];
	avBitmapInfo avbi;
	Item renderMsgPort;
	Item endMsgPort;
	CCB myCCB;
	int32 renderTo;

	char *serverIP;
	char *iisIP;
	char *clientIP;
	int32 port;
	Stream *fp;

} Global;

#ifdef ROD_1

#include "event.h"					//  REP add include file
#include "TEA.h"					//  REP add include file
#include "Rod_Init.c"

#endif

bool DMAmode = FALSE;
bool videoInvideo = true;
bool PhaseIn = FALSE;
bool PhaseOut = FALSE;
Point corners1[4],corners2[4];
Point spare;
bool hold = TRUE;
Coord bottomleftxvalue, bottomleftyvalue, topleftxvalue, topleftyvalue;
int	rotationState = 0;

void CleanUp(Global *g)
{
	int32 i;

	if (g->stream)
	{
		avCloseStreamFile(g->stream);
	}

#ifdef ROD_1
 	while ((gThreadInfo.MessageBits & TEA_TRANSITION_DONE) == 0) ;
	gTEAmsg = WaitPort(gThreadInfo.TEA_ParentPort, 0);
	TEA_FinishThread(&gThreadInfo);
#endif

	if (g->screenGroup > 0)
	{
		RemoveScreenGroup(g->screenGroup);
		DeleteScreenGroup(g->screenGroup);
	}

	for (i = 0; i < CAPTUREBUFS; i++)
	{
		if (g->cBitmapsI[i] > 0)
		{
			DeleteItem(g->cBitmapsI[i]);
		}
	}
	
	for (i = 0; i < VIDEOBUFS; i++)
	{
		if (g->video[i])
		{
			FreeMem(g->video[i], BUFSIZE);
		}
	}

	for (i = 0; i < CAPTUREBUFS; i++)
	{
		if (g->capture[i])
		{
			FreeMem(g->capture[i], BUFSIZE);
		}
	}

	if (g->endMsgPort)
	{
		DeleteMsgPort(g->endMsgPort);
	}

	if (g->renderMsgPort)
	{
		DeleteMsgPort(g->renderMsgPort);
	}
	
	if (g->streamer)
	{
		avRemoveStreamer(g->streamer);
	}
}

void EraseScreen(Bitmap *bm, int32 pages, int32 fill)
{
	Item VRAMIOReq;
	
	VRAMIOReq = GetVRAMIOReq();
	SetVRAMPages(VRAMIOReq, bm->bm_Buffer, fill, pages, -1);
	DeleteItem(VRAMIOReq);
}

#define STT_FILENAME "/remote/system/stt/stt.config"

static char *parse_file( Stream *fp )
{
  static char buff[128];
  char *p, c;

  p = buff;
  buff[0] = 0;

  /* get the first character */
  if( !ReadDiskStream(fp, &c, 1) ) return( buff );

  while( c != '\r' && c != '#' ) {
    *p++ = c;
    if( !ReadDiskStream(fp, &c, 1) )
      c = '\n';
  }

  if( c == '#' )
    while( ReadDiskStream(fp, &c, 1) && c != '\r' )
      ;

  *p = 0; /* null terminate */
  return( buff );
}

uint32 inet_addr( char *cp )
{
  register uint32 val, base, n;
  register char c;
  uint32 parts[4], *pp = parts;
  
//  SPRT(("inet_addr: %s\n",cp));
  
 again:
  /*
   * Collect number up to ``.''.
   * Values are specified as for C:
   * 0x=hex, 0=octal, other=decimal.
   */
  val = 0; base = 10;
  if (*cp == '0') {
    if (*++cp == 'x' || *cp == 'X')
      base = 16, cp++;
    else
      base = 8;
  }
  while ( (c = *cp) != 0 ) {
    if (isdigit(c)) {
      val = (val * base) + (uint32)(c - '0');
      cp++;
      continue;
    }
    if (base == 16 && isxdigit(c)) {
      val = (val << 4) + (uint32)(c + 10 - (islower(c) ? 'a' : 'A'));
      cp++;
      continue;
    }
    break;
  }
  if (*cp == '.') {
    /*
     * Internet format:
     *	a.b.c.d
     *	a.b.c	(with c treated as 16-bits)
     *	a.b	(with b treated as 24 bits)
     */
    if (pp >= parts + 4)
      return ( 0 );
    *pp++ = val, cp++;
    goto again;
  }
  /*
   * Check for trailing characters.
   */
  if (*cp && !isspace(*cp))
    return ( 0 );
  *pp++ = val;
  /*
   * Concoct the address according to
   * the number of parts specified.
   */
  n = pp - parts;
  switch (n) {
    
  case 1:				/* a -- 32 bits */
    val = parts[0];
    break;
    
  case 2:				/* a.b -- 8.24 bits */
    val = (parts[0] << 24) | (parts[1] & 0xffffff);
    break;
    
  case 3:				/* a.b.c -- 8.8.16 bits */
    val = (parts[0] << 24) | ((parts[1] & 0xff) << 16) |
      (parts[2] & 0xffff);
    break;
    
  case 4:				/* a.b.c.d -- 8.8.8.8 bits */
    val = (parts[0] << 24) | ((parts[1] & 0xff) << 16) |
      ((parts[2] & 0xff) << 8) | (parts[3] & 0xff);
    break;
    
  default:
    return ( 0 );
  }
//  val = htonl(val);
//  SPRT(("inet_addr: val = 0x%lx\n",val));
  return (val);
}

int main(int argc, char *argv[])
{
	Global *glob;
	Err err;
	int32 i,ii,loopcount=0;
	TagArg bmTags[] =
	{
		{CBM_TAG_WIDTH, (void *)SC_WIDTH},
		{CBM_TAG_HEIGHT, (void *)SC_HEIGHT},
		{CBM_TAG_BUFFER, NULL},
		{CBM_TAG_DONE, NULL},
	};
	TagArg scTags[] =
	{
		{CSG_TAG_BITMAPBUF_ARRAY, NULL},
		{CSG_TAG_SCREENCOUNT, (void *)2},
		{CSG_TAG_SCREENHEIGHT, (void *)SC_HEIGHT},
		{TAG_END, NULL},
	};
	Point directions[4];
	Item msgI;
	Message *msg;
	int32 pages;
	int32 pageSize;
	JoyPadState jp;
	bool alive = TRUE;
	bool blend = false;
	bool cycle = FALSE;
	bool useAV = FALSE;
	int32 bgnds[] =
	{
		0x00000000, 
		0xffffffff,
		0x7c007c00,
		0x03e003e0,
		0x001f001f,
		0x7fe07fe0,
		0x7c1f7c1f,
		0x03ff03ff,
	};
	int32 bgnd = 0;
	int32 bgndColour = 0;
	int32 countdown;
	uint32 signalMask;
	uint32 signals;
	Rect box;
	TagArg cva[] =
	{
		AV_TAG_CVA_SIZE, NULL,
		AV_TAG_CVA_CORNERS, NULL,
		TAG_END, NULL,
	};
	Stream *fp;
	char *streamerName = AV_CABLE_ANALOG_NAME;
	char *fileName = "channel1";
	char sttServer[16], sttiis[16], client[16];

printf("HERE we go......\n\n\n\n");

	if ((argc > 1) && (*argv[1] == '?'))
	{
		printf("Bounce:\n");
		printf("-c for avCableDigitalStreamer\n");
		printf("-s [name] for streamer name\n");
		printf("-f [name] for filename\n");
		exit(0);
	}

	for (i = 1; i < argc; i++)		
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			streamerName = AV_CABLE_DIGITAL_NAME;
		}
		if (strcmp(argv[i], "-s") == 0)
		{
			streamerName = argv[++i];
		}
		if (strcmp(argv[i], "-f") == 0)
		{
			fileName = argv[++i];
		}
		if (strcmp(argv[i], "-x") == 0)
		{
			DMAmode = true;
		}
	}

	if ((glob = (Global *)AllocMem(sizeof(Global), (MEMTYPE_ANY | MEMTYPE_FILL | 0))) == NULL)
	{
		printf("No memory\n");
		exit(0);
	}
	

	glob->serverIP = "192.33.138.1";
	glob->iisIP = "192.33.138.1";
	glob->clientIP = "192.33.138.57";
	glob->port = 3001;
	fp = OpenDiskStream(STT_FILENAME, 0);
	if (fp)
	{
		strcpy(sttiis, parse_file(fp));
		glob->iisIP = sttiis;
		strcpy(sttServer, parse_file(fp));
		glob->serverIP = sttServer;
		strcpy(client, parse_file(fp));
		glob->clientIP = client;
		glob->port = strtoul(parse_file(fp), NULL, 10);
		printf("From stt.config:\n");
		printf("iis = %s\n", glob->iisIP);
		printf("server = %s\n", glob->serverIP);
		printf("client = 0x%lx\n", glob->clientIP);
		printf("port = %ld\n", glob->port);
		CloseDiskStream(fp);
	}
	else
	{
		printf("Error opening %s\n", STT_FILENAME);
	}

	if ((err = OpenGraphicsFolio()) < 0)
	{
		PrintfSysErr(err);
		exit(0);
	}

	if ((err = OpenAVFolio()) < 0)
	{
		PrintfSysErr(err);
		exit(0);
	}

	if ((glob->streamer = avGetStreamer(streamerName)) < 0)
	{
		PrintfSysErr(err);
		CleanUp(glob);
		exit(0);
	}

	if ((glob->renderMsgPort = CreateMsgPort(NULL, 0, 0)) < 0)
	{
		PrintfSysErr(glob->renderMsgPort);
		CleanUp(glob);
		exit(0);
	}

	if ((glob->endMsgPort = CreateMsgPort(NULL, 0, 0)) < 0)
	{
		PrintfSysErr(glob->endMsgPort);
		CleanUp(glob);
		exit(0);
	}

	/* Allocate our display buffers.
	 * We need two for double buffering the display,
	 * and two for capturing the video.
	 */
	for (i = 0; i < CAPTUREBUFS; i++)
	{
		if ((glob->capture[i] = (ubyte *)AllocMem(BUFSIZE, (MEMTYPE_DMA | MEMTYPE_CEL | MEMTYPE_FILL | 0))) == NULL)
		{
			printf("No memory for capture buffers\n");
			CleanUp(glob);
			exit(0);
		}
	}

	for (i = 0; i < VIDEOBUFS; i++)
	{
		if ((glob->video[i] = (ubyte *)AllocMem(BUFSIZE, (MEMTYPE_VRAM | MEMTYPE_CEL | MEMTYPE_FILL | 0))) == NULL)
		{
			printf("No memory for video buffers\n");
			CleanUp(glob);
			exit(0);
		}
	}

	/* Calculate the number of pages to clear */
	pageSize = GetPageSize(MEMTYPE_VRAM);
	pages = ((BUFSIZE + pageSize - 1) / pageSize);

	/* Create the Capture Bitmap Items */
	for (i = 0; i < CAPTUREBUFS; i++)
	{
		bmTags[2].ta_Arg = (void *)glob->capture[i];
		if ((glob->cBitmapsI[i] = CreateBitmap(bmTags)) < 0)
		{
			printf("Could not create bitmap\n");
			PrintfSysErr(glob->cBitmapsI[i]);
			CleanUp(glob);
			exit(0);
		}
		glob->cBitmaps[i] = (Bitmap *)LookupItem(glob->cBitmapsI[i]);
	}

	/* Create the screens */
	scTags[0].ta_Arg = (void *)&glob->video[0];
	if ((glob->screenGroup = CreateScreenGroup(&glob->screensI[0], scTags)) < 0)
	{
		printf("Could not create screengroup\n");
		PrintfSysErr(glob->screenGroup);
		CleanUp(glob);
		exit(0);
	}
	for (i = 0; i < VIDEOBUFS; i++)
	{
		glob->screens[i] = (Screen *)LookupItem(glob->screensI[i]);
		glob->vBitmaps[i] = glob->screens[i]->scr_TempBitmap;
		glob->vBitmapsI[i] = glob->vBitmaps[i]->bm.n_Item;
	}

	/* Create the avBitmapInfo() we will pass to the AVFolio */
	glob->avbi.avbi_bitmapCount = CAPTUREBUFS;
	glob->avbi.avbi_bpp = 16;
	for (i = 0; i < CAPTUREBUFS; i++)
	{
		glob->avbi.avbi_bitmaps[i] = glob->cBitmaps[i];
	}
	AddScreenGroup(glob->screenGroup, NULL);
	DisplayScreen(glob->screensI[0], 0);
	glob->renderTo = 1;

	/* Set up the corners */
	corners1[0].pt_X = 0;
	corners1[0].pt_Y = 0;
	corners1[1].pt_X = (SC_WIDTH - 1);
	corners1[1].pt_Y = 0;
	corners1[2].pt_X = (SC_WIDTH - 1);
	corners1[2].pt_Y = (SC_HEIGHT - 1);
	corners1[3].pt_X = 0;
	corners1[3].pt_Y = (SC_HEIGHT - 1);

#if 0
	corners2[0].pt_X = 5 * SC_WIDTH / 8;
	corners2[0].pt_Y = 10;
	corners2[1].pt_X = (SC_WIDTH - 1);
	corners2[1].pt_Y = 10;
	corners2[2].pt_X = (SC_WIDTH - 1);
	corners2[2].pt_Y = 3 * SC_HEIGHT / 8;
	corners2[3].pt_X = 5 * SC_WIDTH / 8;
	corners2[3].pt_Y = 3 * SC_HEIGHT / 8;
#endif 

	corners2[0].pt_X = 5 * SC_WIDTH / 8;
	corners2[0].pt_Y = 0;
	corners2[1].pt_X = 120 + corners2[0].pt_X;
	corners2[1].pt_Y = corners2[0].pt_Y;
	corners2[2].pt_X = 120 + corners2[0].pt_X;
	corners2[2].pt_Y = 90 + corners2[0].pt_Y;
	corners2[3].pt_X = corners2[0].pt_X;
	corners2[3].pt_Y = 90 + corners2[0].pt_Y;

	box.rect_XLeft = 0;
	box.rect_YTop = 0;
	box.rect_XRight = (IMAGE_WIDTH - 1);
	box.rect_YBottom = (IMAGE_HEIGHT - 1);

	/* Set up the directions */
	for (i = 0; i < 4; i++)
	{
		directions[i].pt_X = 1;
		directions[i].pt_Y = 1;
	}

	glob->myCCB.ccb_Width = SC_WIDTH;
	glob->myCCB.ccb_Height = SC_HEIGHT;
	glob->myCCB.ccb_Flags = (CCB_LAST | CCB_SPABS | CCB_PPABS | CCB_LDSIZE | CCB_LDPRS | CCB_LDPPMP |
		                     CCB_CCBPRE | CCB_YOXY | CCB_ACSC | CCB_ACW | CCB_ACCW | 
	                         PMODE_ONE | CCB_BGND | CCB_NOBLK);
	glob->myCCB.ccb_PIXC = ((((1 << PPMPC_MF_SHIFT) | (1 << PPMPC_SF_SHIFT)) << PPMP_1_SHIFT) |
	                        ((1 << PPMPC_MF_SHIFT) | (1 << PPMPC_SF_SHIFT)));
	glob->myCCB.ccb_PRE0 = (PRE0_BGND | (((SC_HEIGHT / 2) - 1) << PRE0_VCNT_SHIFT) | PRE0_LINEAR | PRE0_BPP_16);
	glob->myCCB.ccb_PRE1 = (((SC_WIDTH - 2) << PRE1_WOFFSET10_SHIFT) | PRE1_LRFORM | (SC_WIDTH - 1));

	/* Open the AVFolio stream */
	if( DMAmode )
	{
		if ((glob->stream = avOpenStreamFileVA(glob->streamer, fileName, glob->endMsgPort,
											  AV_TAG_OSF_BITMAPS, (void *)&glob->avbi,
											  AV_TAG_OSF_I_DISPLAY, (void *)glob->renderMsgPort,
											  AV_TAG_OSF_CABLED_SERVERIP, (void *)glob->serverIP,
											  AV_TAG_OSF_CABLED_CLIENTIP, (void *)glob->clientIP,
											  AV_TAG_OSF_CABLED_PORT, (void *)glob->port,
											  AV_TAG_OSF_CABLED_IISIP, (void *)glob->iisIP,
									          AV_TAG_OSF_CABLED_VDMA, 0,
											  TAG_END)) < 0)
		{
			printf("Could not open the stream\n");
			PrintfSysErr(glob->stream);
			CleanUp(glob);
			exit(0);
		}
	}
	else
	{
		if ((glob->stream = avOpenStreamFileVA(glob->streamer, fileName, glob->endMsgPort,
											  AV_TAG_OSF_BITMAPS, (void *)&glob->avbi,
											  AV_TAG_OSF_I_DISPLAY, (void *)glob->renderMsgPort,
											  AV_TAG_OSF_CABLED_SERVERIP, (void *)glob->serverIP,
											  AV_TAG_OSF_CABLED_CLIENTIP, (void *)glob->clientIP,
											  AV_TAG_OSF_CABLED_PORT, (void *)glob->port,
											  AV_TAG_OSF_CABLED_IISIP, (void *)glob->iisIP,
											  TAG_END)) < 0)
		{
			printf("Could not open the stream\n");
			PrintfSysErr(glob->stream);
			CleanUp(glob);
			exit(0);
		}
	}

#ifdef ROD_1
 	InitTEA(glob);
	gMovieStarted = false;
	
	gCurrentEffect = 0;
	gEffectCnt = 0;
	
	TEA_CallThread(EffectProcs[gCurrentEffect++].EffectProcPtr, 0xFFFFFFFF, &CopyData, &gThreadInfo, NULL, NULL );
	gThreadInfo.MessageBits |= TEA_DISPLAYSCREEN;
	gThreadInfo.MessageBits &= ~TEA_WANT_HANDSHAKING;

#endif
	
	avPlayStream(glob->stream, AV_RATE_NORMAL, FALSE);

	countdown = 6;
	signalMask = ((MsgPort *)LookupItem(glob->renderMsgPort))->mp_Signal;
	signalMask |= ((MsgPort *)LookupItem(glob->endMsgPort))->mp_Signal;

	if (blend)
	{
		glob->myCCB.ccb_PIXC = (((PPMPC_SF_2 | PPMPC_2S_CFBD | PPMPC_2D_2) << PPMP_1_SHIFT) |
								(PPMPC_SF_2 | PPMPC_2S_CFBD | PPMPC_2D_2));
	}			
	else
	{
		glob->myCCB.ccb_PIXC = ((((1 << PPMPC_MF_SHIFT) | (1 << PPMPC_SF_SHIFT)) << PPMP_1_SHIFT) |
								((1 << PPMPC_MF_SHIFT) | (1 << PPMPC_SF_SHIFT)));
	}						

	while (alive && (signals = WaitSignal(signalMask)))
	{
		Bitmap *bm;

#ifdef ROD_1
		if (gThreadInfo.MessageBits & TEA_TRANSITION_DONE)
		{
			int32 retValue;

			if (++gEffectCnt >= 2)
			{
				while ((gThreadInfo.MessageBits & TEA_TRANSITION_DONE) == 0) ;
				gTEAmsg = WaitPort(gThreadInfo.TEA_ParentPort, 0);
			    TEA_FinishThread(&gThreadInfo);
				TEA_CallThread(EffectProcs[gCurrentEffect++].EffectProcPtr, 0xFFFFFFFF, &CopyData, &gThreadInfo, NULL, NULL );
				gThreadInfo.MessageBits |= TEA_DISPLAYSCREEN;
				gThreadInfo.MessageBits &= ~TEA_WANT_HANDSHAKING;
				
				gEffectCnt = 0;
			}
			else {
					gTEAmsg = WaitPort(gThreadInfo.TEA_ParentPort, 0);
					retValue = SendSmallMsg(gThreadInfo.TEA_ChildPort, gThreadInfo.TEA_ParentMsg, TEA_REPEAT_TRANSITION, 0xEAEA);
					FAILOSERR("Cannot send message", retValue);
					gTEAmsg = WaitPort(gThreadInfo.TEA_ParentPort, 0);
				 }
			gMovieStarted = false;
			
		}
#endif


		msgI = GetMsg(glob->endMsgPort);
		if (msgI > 0)
		{
			/* End of the stream */
			printf("@@@ End of the stream\n");
			ReplyMsg(msgI, 0, NULL, 0);
			alive = FALSE;
			break;
		}

		msgI = GetMsg(glob->renderMsgPort);
		if (msgI <= 0)
		{
			ReplySmallMsg(msgI, 0, 0, 0);
			continue;
		}
		msg = (Message *)LookupItem(msgI);

		/* This is a Small message, with msg_DataSize is the Bitmap * to use. */
		bm = (Bitmap *)msg->msg_DataSize;

		GetJoyPad(&jp, 1);
		
		if (jp.xBtn)
		{
			alive = FALSE;
		}
		if (jp.cBtn)
		{
			if ( ( PhaseIn ) || (PhaseOut) )
			{
				// do nothing as you don't want to interrupt the special effect
			}
			else
			{
				videoInvideo = !videoInvideo;
				if( videoInvideo )
				{
					PhaseIn = TRUE;
					PhaseOut = FALSE;
					bottomleftxvalue = corners2[3].pt_X;
					bottomleftyvalue = corners2[3].pt_Y;
					corners2[3].pt_X = corners2[1].pt_X;
					corners2[3].pt_Y = corners2[1].pt_Y;
				}
				else
				{
					PhaseIn = FALSE;
					PhaseOut = TRUE;
					videoInvideo = TRUE;	// this is set to false after the phaseout
					topleftxvalue = corners2[0].pt_X;
					topleftyvalue = corners2[0].pt_Y;
				}
			}
		}
		if (jp.bBtn)
		{
			spare.pt_X = corners2[0].pt_X;	
			spare.pt_Y = corners2[0].pt_Y;	
			corners2[0].pt_X = corners2[1].pt_X;
			corners2[0].pt_Y = corners2[1].pt_Y;
			corners2[1].pt_X = corners2[2].pt_X;
			corners2[1].pt_Y = corners2[2].pt_Y;
			corners2[2].pt_X = corners2[3].pt_X;
			corners2[2].pt_Y = corners2[3].pt_Y;
			corners2[3].pt_X = spare.pt_X;
			corners2[3].pt_Y = spare.pt_Y;
#if 0
			blend = !blend;
			if (blend)
			{
				glob->myCCB.ccb_PIXC = (((PPMPC_SF_2 | PPMPC_2S_CFBD | PPMPC_2D_2) << PPMP_1_SHIFT) |
				                        (PPMPC_SF_2 | PPMPC_2S_CFBD | PPMPC_2D_2));
			}			
			else
			{
				glob->myCCB.ccb_PIXC = ((((1 << PPMPC_MF_SHIFT) | (1 << PPMPC_SF_SHIFT)) << PPMP_1_SHIFT) |
				                        ((1 << PPMPC_MF_SHIFT) | (1 << PPMPC_SF_SHIFT)));
			}						
#endif
		}
		if (jp.aBtn)
		{
			hold = !hold;
		}

		if(hold)
		{
			for (i = 0; i < 4; i++)
			{
				directions[i].pt_X = 0;
				directions[i].pt_Y = 0;
			}
		}

		if (jp.rightArrow)
		{
			for (i = 0; i < 4; i++)
			{
				directions[i].pt_X = 1;
			}
		}
		if (jp.leftArrow)
		{
			for (i = 0; i < 4; i++)
			{
				directions[i].pt_X = -1;
			}
		}
		if (jp.upArrow)
		{
			for (i = 0; i < 4; i++)
			{
				directions[i].pt_Y = -1;
			}
		}
		if (jp.downArrow)
		{
			for (i = 0; i < 4; i++)
			{
				directions[i].pt_Y = 1;
			}
		}

		/* Move the corners next time. */
		if (!hold)
		{
			for (i = 0; i < 4; i++)
			{
				int32 dir;
				corners2[i].pt_X += directions[i].pt_X;
				corners2[i].pt_Y += directions[i].pt_Y;

	
				/* bounds check */
				if (corners2[i].pt_X < 0)
				{
					corners2[i].pt_X = 0;
				}
	
				if (corners2[i].pt_X >= SC_WIDTH)
				{
					corners2[i].pt_X = (SC_WIDTH - 1);
				}
	
				if (corners2[i].pt_Y < 0)
				{
					corners2[i].pt_Y = 0;
				}
	
				if (corners2[i].pt_Y >= SC_HEIGHT)
				{
					corners2[i].pt_Y = (SC_HEIGHT - 1);
				}
			}
			bottomleftxvalue += directions[3].pt_X;
			bottomleftyvalue += directions[3].pt_Y;
			topleftxvalue += directions[0].pt_X;
			topleftyvalue += directions[0].pt_Y;
		}

		if (!blend)
		{
	//		EraseScreen(glob->vBitmaps[glob->renderTo], pages, bgndColour);
		}
		glob->myCCB.ccb_SourcePtr = (CelData *)bm->bm_Buffer;

#ifdef ROD_1
		if (gMovieStarted == FALSE)
		{
			int32 retValue;
			
			gMovieStarted = true;
						
			retValue = SendSmallMsg(gThreadInfo.TEA_ChildPort, gThreadInfo.TEA_ParentMsg, TEA_CONTINUE_TRANSITION, 0xEAEA);
			FAILOSERR("Cannot send message", retValue);
		}				
#endif

		if (useAV)  
		{
			avChangeVideoAttribute(glob->stream, NULL, 0, cva);
		}
		else
		{
			MapCel(&glob->myCCB, &corners1[0]);
//			DrawCels(glob->vBitmapsI[glob->renderTo], &glob->myCCB);
			if (false  &&  videoInvideo)
			{
	//			hold = true;
				MapCel(&glob->myCCB, &corners2[0]);
				DrawCels(glob->vBitmapsI[glob->renderTo], &glob->myCCB);
				if(PhaseIn)
				{
#ifdef FLIP_ROTATE
					// flipped + 90degree anticlockwise
					if ( (corners2[1].pt_X > bottomleftxvalue) && (corners2[1].pt_Y < bottomleftyvalue) )
#else
					if ( (corners2[3].pt_X > bottomleftxvalue) && (corners2[3].pt_Y < bottomleftyvalue) )
#endif
					{
#ifdef FLIP_ROTATE
						// flipped + 90degree anticlockwise
						corners2[1].pt_X -= 2;
						corners2[1].pt_Y += 2;
#else
						corners2[3].pt_X -= PHASE_IN_SPEED;
						corners2[3].pt_Y += PHASE_IN_SPEED;
#endif
					}
					else
					{
						if ( corners2[3].pt_X > bottomleftxvalue )
						{
							corners2[3].pt_X -= PHASE_IN_SPEED;
						}
						else
						{
							if ( corners2[3].pt_Y < bottomleftyvalue )
							{
								corners2[3].pt_Y += PHASE_IN_SPEED;
							}
							else
							{
								corners2[3].pt_X = bottomleftxvalue;
								corners2[3].pt_Y = bottomleftyvalue;
								PhaseIn = FALSE;
							}
						}
					}
				}
				else
				{
					if (PhaseOut)
					{
						if ( (corners2[2].pt_X > topleftxvalue) && (corners2[2].pt_Y > topleftyvalue) )
						{
							corners2[2].pt_X -= PHASE_OUT_SPEED;
							corners2[2].pt_Y -= PHASE_OUT_SPEED;
						}
						else
						{
							if (corners2[2].pt_X > topleftxvalue)
							{
								corners2[2].pt_X -= PHASE_OUT_SPEED;
							}
							else
							{
								if (corners2[2].pt_Y > topleftyvalue)
								{
									corners2[2].pt_Y -= PHASE_OUT_SPEED;
								}
								else
								{
									corners2[2].pt_X = topleftxvalue;
									corners2[2].pt_Y = topleftyvalue;
									videoInvideo = FALSE;
									PhaseOut = FALSE;
									corners2[1].pt_X = 120 + corners2[0].pt_X;
									corners2[1].pt_Y = corners2[0].pt_Y;
									corners2[2].pt_X = 120 + corners2[0].pt_X;
									corners2[2].pt_Y = 90 + corners2[0].pt_Y;
									corners2[3].pt_X = corners2[0].pt_X;
									corners2[3].pt_Y = 90 + corners2[0].pt_Y;
									hold = TRUE;
								}
							}
						}
					}
				}
			}
		}
		
#ifndef ROD_1
  		DisplayScreen(glob->screensI[glob->renderTo], 0);
#endif

		if (!blend)
		{
			glob->renderTo++;
			glob->renderTo %= VIDEOBUFS;
		}

		ReplySmallMsg(msgI, 0, 0, 0);
	}
	CleanUp(glob);
	exit(0);
}


