#include "scaffold.h"
#include "strings.h"

#include "TSReporter.h"
#include "stdio.h"

#define	PRINTIT Report

extern char * gTestName;
extern char * gProgName;
extern int32 gFlag;
extern int32 gTestLevel;
extern int32 gTestType;


int32	gCountUNRESOLVED;
int32	gCountUNTESTED;
int32	gCountPASSED;
int32	gCountFAILED;

/*********************************************************************************************\
Routine:TestComment()

Synopsis: void TestComment(char * InString)

What:	TestComment() outputs a string to standard output.
		
How:	
		
Deps:	PRINTIT().

Notes:	need to be able to deal with dynamic arguments.

Who:	Written By Eric Matsuno  4-13-94.
\*********************************************************************************************/
void TestComment(char * InString,...)
{
	char * CommentString = "-----comment: ";
	va_list	ap;

	va_start(ap, InString);
	
	TSRptvPipeLog(CommentString, ap  );
	TSRptvPipeScreen( CommentString, ap );

	TSRptvPipeLog(  InString, ap  );
	TSRptvPipeScreen( InString, ap );
	va_end( ap );
}

/*********************************************************************************************\
Routine:DebugComment()

Synopsis: void DebugComment(char * InString)

What:	DebugComment() outputs a string to standard output in debug mode.
		** CURRENTLY DOES NOTHING **.
		
How:	
		
Deps:	PRINTIT().

Who:	Written By Eric Hegstrom  5-23-94.

Revisions:	Added functionality   Eric Matsuno 5-31-94
\*********************************************************************************************/
void DebugComment(char * InString,...)
{
	va_list	ap;
	va_start(ap, InString);
	TSRptvPipeScreen( InString, ap );

}

/*********************************************************************************************\
Routine:FlagSet()

Synopsis: void FlagSet(int32 Flag)

What:	FlagSet() set's the Test's result flag to Flag.
		
How:	By hierarchical standards, Setflag set's the appropriate Flag.
		
Deps:	vPipeData().

Notes:	None

Who:	Written By Eric Matsuno  3-10-94.
\*********************************************************************************************/

void FlagSet(int32 Flag)
{

	if (Flag == UNRESOLVED)
		gFlag = UNRESOLVED;
	else if ((Flag == FAILED) && (gFlag < FAILED))
			gFlag = FAILED;
}	

/*********************************************************************************************\
Routine:SetTestResult()

Synopsis: void SetTestResult(int32 Flag,char * InString)

What:	SetTestResult() set's the Test's result flag to Flag, and outputs InString.
		
How:	By hierarchical standards, Setflag set's the appropriate Flag.
		
Deps:	vPipeData().

Notes:	None

Who:	Written By Eric Matsuno  3-10-94.
\*********************************************************************************************/

void SetTestResult(int32 Flag,char * InString,...)
{
//	char * SetString = "----- ";
	char SetString[255];
	char * ResultString;
	va_list	ap;

	FlagSet(Flag);
	

	if (Flag == UNRESOLVED)
		ResultString = "unresolved: ";
	else if (Flag == FAILED)
		ResultString = "failed: ";
	else 
		return;
		
	strcpy(SetString,"----- ");
	strcat(SetString,ResultString);
	
	va_start(ap, InString);
	
	TSRptvPipeLog(SetString, ap  );
	TSRptvPipeScreen( SetString, ap );

	TSRptvPipeLog(  InString, ap  );
	TSRptvPipeScreen( InString, ap );
	
	va_end( ap );

}

/*********************************************************************************************\
Routine:InitTestProgram()

Synopsis: void InitTestProgram(void)

What:	This routine initializes the test environment 
		
How:	It simply calls the appropriate initialization routines for all of the necessary
		init objects.
		
Deps:	ReportInit().

Notes:	None

Who:	Written By Eric Matsuno  3-10-94.
\*********************************************************************************************/
void InitTestProgram(void)
{
	ReportInit();	
	
	gCountUNRESOLVED	= 0;
	gCountUNTESTED		= 0;
	gCountPASSED		= 0;
	gCountFAILED		= 0;

}

/*********************************************************************************************\
Routine:BlockEnter()

Synopsis: void BlockEnter(void)

What:	This routine initializes the test block environment (not the program environment)
		and outputs the beginning of the test block.
		Initializes the following:  gflag to UNTESTED
		
How:	It simply sets gflag to UNTESTED, and outputs the beginning of the block
		
Deps:	PRINTIT().

Notes:	None

Who:	Written By Eric Matsuno  3-10-94.
\*********************************************************************************************/
void BlockEnter(void)
{
char * level;
char * type;

gFlag = UNTESTED;

	switch(gTestLevel)
	{
		case BREADTH:	level = "Breadth"; break;
		case DEPTH:		level = "Depth"; break;
		case HIGH_RISK:	level = "High Risk"; break;
		default:	level = "Unknown"; break;
	}
	
	switch(gTestType)
	{
		case INTERACTIVE:	type = "Interactive"; break;
		case AUTOMATED:		type = "Automated"; break;
		default:		type = "Unknown"; break;
	}	
		
// PRINTIT("------------------BEGINNING %s------------------\n",gTestName);


}





void SetPassed(void)
{
	if ((gFlag == UNTESTED) && (gTestType == AUTOMATED))
		gFlag = PASSED;
}
/*********************************************************************************************\
Routine:BlockExit()

Synopsis: void AnyFail(void)

What:	AnyFail() Outputs the block's flag status.
		
How:	
		
Deps:	PRINTIT().

Notes:	None

Who:	Written By Eric Matsuno  3-10-94.
\*********************************************************************************************/
void BlockExit(void)
{
char * ResultString;

	SetPassed();

	if (gFlag == UNRESOLVED)
	{
		ResultString = "UNRESOLVED";
		gCountUNRESOLVED++;
	}
	else if (gFlag == FAILED)
	{
		ResultString = "FAILED    ";
		gCountFAILED++;
	}
	else if (gFlag == PASSED)
	{
		ResultString = "PASSED    ";
		gCountPASSED++;
	}
	else if (gTestType == INTERACTIVE)
		ResultString = "INTERACTIVE";
	else if (gFlag == UNTESTED)
	{
		ResultString = "UNTESTED  ";
		gCountUNTESTED++;
	}
	
	PRINTIT("%s   --  %s\n", ResultString, gTestName);
}

/*********************************************************************************************\
Routine:FinalMessage()

Synopsis: void InitTestProgram(void)

What:	This routine terminates the test environment 
		
How:	It simply calls the appropriate initialization routines for all of the necessary
		init objects.
		
Deps:	ReportTerm().

Notes:	None

Who:	Written By Eric Hegstrom  5-20-94.
\*********************************************************************************************/
void FinalMessage(void)
{

	PRINTIT("\n\n\n");
	PRINTIT("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n");
	PRINTIT("          SUMMARY\n\n");
	PRINTIT(" UNTESTED       %d\n", gCountUNTESTED);
	PRINTIT(" UNRESOLVED     %d\n\n", gCountUNRESOLVED);
	PRINTIT(" FAILED         %d\n", gCountFAILED);
	PRINTIT(" PASSED         %d\n\n", gCountPASSED);
	PRINTIT("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n");

	ReportTerm();	
}
