
// 	------------------------------------------------------------------------------------------
//																	Enable / disable switches
#include "AVtest_Enable.h"

// 	------------------------------------------------------------------------------------------
//														 Globals required by output formatter
#include "scaffold.h"
#include "Globals.h"

// 	------------------------------------------------------------------------------------------
//																					Includes
#include "AVtest_includes.h"

// 	------------------------------------------------------------------------------------------
//																					  Macros
#include "AVtest_macros.h"

// 	------------------------------------------------------------------------------------------
//																			Global variables
extern int optind;
extern char *optarg;

// 	------------------------------------------------------------------------------------------
//																		   Function typedefs
int32 getopt(int32 nargc, char * const *nargv, const char *ostr );

// 	------------------------------------------------------------------------------------------
//																		Function definitions
#include "AVtest_functions.c"

// 	------------------------------------------------------------------------------------------
//																					 main()
int main(int argc, char *argv[])
{
#include "AVtest_main()Variables"
#ifdef ENABLE_TESTS
#include "InitDepthTest.h"									// Initialize output formatter
#endif
#include "AVtest_initialization.c"

	/* Create a message port to be signalled when the stream has finished. */
	endMsgPort = CreateMsgPort(NULL, 0, 0);
	DPRT(("@@@ endMsgPort = 0x%lx\n", endMsgPort));

	/* Get the appropriate streamer */
	kprintf("Opening Cable streamer\n");
// 	------------------------------------------------------------------------------------------
//																		 avGetStreamer tests
	#include "AVtest_GetStreamer.c"
//
// 	------------------------------------------------------------------------------------------
	streamer = avGetStreamer(streamerName);
	kprintf("streamer = 0x%lx\n", streamer);

// 	------------------------------------------------------------------------------------------
//																		   OpenAVfolio tests
	#include "AVtest_OpenAVfolio.c"
//
// 	------------------------------------------------------------------------------------------
	/* Were we succesful? */
	if (streamer > 0)
	{
		/* Get the abilities. This tells us what the streamer is capable
		 * of, so we know what control options to give the user.
		 *
		 * Each ta_Arg is a pointer to a bool variable, which the AVFolio
		 * will set TRUE or FALSE depending on whether the streamer
		 * supports the respective ability.
		 *
		 */
// 	------------------------------------------------------------------------------------------
//																avGetStreamerAbilities tests
		#include "AVtest_GetStreamerAbilities.c"
//
// 	------------------------------------------------------------------------------------------
		for (i = 0; i < ((sizeof(getAbilities) / sizeof(TagArg)) - 1); i++)
		{
			getAbilities[i].ta_Arg = (void *)&abilities[i];
		}
		avGetStreamerAbilities(streamer, getAbilities);
		for (i = 0; i < (sizeof(getAbilities) / sizeof(TagArg)); i++)
		{
			DPRT(("Able %ld = %s\n", i, (abilities[i] ? "TRUE" : "FALSE")));
		}

		/* If we are allocating the screen, do so. */
		if (allocateScreen)
		{
			/* 16 bit NYI */
			screenGroupI = Open24BitGraphics(&sc, 2);
			if (screenGroupI > 0)
			{
				printf("@@@ Allocated screen\n");
				/* Set up the avBitmapInfo so we can tell the AVFolio
				 * what the screen and bitmaps are.
				 */
				avbi.avbi_bitmapVisible = 0;
				avbi.avbi_bitmapCount = 4;
				avbi.avbi_bpp = 24;
				for (i = 0; i < 4; i++)
				{
					avbi.avbi_bitmaps[i] = sc.sc_Bitmaps[i];
					avbi.avbi_ScreenI[i] = sc.sc_Screens[i];
				}
			}
			else
			{
				PrintfSysErr(screenGroupI);
				loop = FALSE;
			}
		}


		/* Here comes the main loop. */
		while (loop)
		{
			if ((!open) && (filename))
			{
				/* We need to open the file to be played. Start by building
				 * the tag list that is passed to avOpenStreamFile().
				 */

				int32 t = 0;
				if (screenGroupI > 0)
				{
					/* Here is the screen I just created */
					osf_tags[t].ta_Tag = AV_TAG_OSF_BITMAPS;
					osf_tags[t].ta_Arg = (void *)&avbi;
					t++;
				}

#include "AVtest_CAMtests.c"

				if (useRPC)
				{
					/* Here's the IP information */
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_SERVERIP;
					osf_tags[t].ta_Arg = (void *)serverIP;
					t++;
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_CLIENTIP;
					osf_tags[t].ta_Arg = (void *)clientIP;
					t++;
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_PORT;
					osf_tags[t].ta_Arg = (void *)port;
					t++;
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_IISIP;
					osf_tags[t].ta_Arg = (void *)iisIP;
					t++;
					if (track != -1)
					{
						sprintf(filename, "%ld", track);
					}
				}

				kprintf("Opening file %s\n", filename);

				if (depth)
				{
					/* If we used -d to set a specific depth, build the
					 * tag here. If not, then the streamer will use its
					 * default depth.
					 */
					osf_tags[t].ta_Tag = ((depth == 16) ?  AV_TAG_OSF_DISPLAY16BPP : AV_TAG_OSF_DISPLAY24BPP);
					t++;
				}
				if (forceDMA)
				{
					/* If we used -v to force DMA mode, build the tag here. */
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_VDMA;
					t++;
				}

				osf_tags[t].ta_Tag = TAG_END;
// 	------------------------------------------------------------------------------------------
//																	  avOpenStreamFile tests
		#include "AVtest_OpenStreamFile.c"			
//
// 	------------------------------------------------------------------------------------------
				stream = avOpenStreamFile(streamer, filename, endMsgPort, osf_tags);
				REPORT_ERR(stream);

				if (open = (stream > 0))
				{
					/* Hooray! It opened. Now just play the stream. */

					err = avPlayStream(stream, AV_RATE_NORMAL, userFeedback);
// 	------------------------------------------------------------------------------------------
//																	 	  avPlayStream tests
		#include "AVtest_PlayStream.c"
//
// 	------------------------------------------------------------------------------------------
					REPORT_ERR(err);
					quick = FALSE;

					/* Set the volume level */
					err = avStartUserFeedbackVA(stream, 
		    			                        AV_TAG_UF_VOLUME, volume,
		    			                        AV_TAG_UF_DURATION_FIELDS, 0,
		                			            TAG_END);
					REPORT_ERR(err);

				}
				else
				{
					/* Bummer dude. */
					loop = FALSE;
				}
			}
	
			if (stream > 0)
			{
				GetJoyPad(&jp, 1);

				/* Hmm. A nasty polling loop! What was I thinking here??? */
				if (endMsg = GetMsg(endMsgPort))
				{
					printf("@@@ End of the stream\n");
					ReplyMsg(endMsg, 0, NULL, 0);
					jp.xBtn = 1;	/* Force the CloseStream code */
				}

				if (jp.xBtn)
				{
					DPRT(("@@@ Closing stream\n"));
					err = avCloseStreamFile(stream);
					REPORT_ERR(err);
					open = FALSE;
					filename = NULL;
					stream = 0;
				}				
	
				/* Note that for volume and mute, we check that the streamer
				 * supports the action.
				 */
				if ((jp.upArrow) && (ABLE(6)))
				{
					DPRT(("@@@ Volume Up\n"));
					if (volume < MAXDSPAMPLITUDE)
					{
						volume += VOLUME_DELTA;
					}
					if (volume > MAXDSPAMPLITUDE)
					{
						volume = MAXDSPAMPLITUDE;
					}
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_VOLUME, volume,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}
	
				if ((jp.downArrow) && (ABLE(6)))
				{
					DPRT(("@@@ Volume Down\n"));
					if (volume > 0)
					{
						volume -= VOLUME_DELTA;
					}
					if (volume < 0)
					{
						volume = 0;
					}
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_VOLUME, volume,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}
	
				if ((jp.leftArrow) && (ABLE(7)))
				{
					/* Mute */
					DPRT(("@@@ mute on\n"));
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_MUTE, TRUE,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}

				if ((jp.rightArrow) && (ABLE(7)))
				{
					/* Mute */
					DPRT(("@@@ mute off\n"));
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_MUTE, FALSE,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}

				if (jp.leftShift)
				{
					/* Temporary hack -- show the buffers on screen */
					err = avStartUserFeedbackVA(stream,
					      AV_TAG_LASTUF, NULL,
					      TAG_END, NULL);
					REPORT_ERR(err);
				}

				if ((jp.rightShift) && (ABLE(5)))
				{
					/* Jump forward five seconds. */
					DPRT(("@@@ forward 5 secs\n"));
					tv.tv_sec = 5;
					err = avSetStreamPositionVA(stream, AV_F_SETSTREAMPOS_DELTA, userFeedback,
					                           AV_TAG_SSP_ABSTIME, (void *)&tv,
					                           TAG_END);
					REPORT_ERR(err);
				}
	
				if ((jp.aBtn) && (track > 0))
				{
					/* Move on to the next numerically ordered stream. This only
					 * makes sense on the cable system where all the streams
					 * are numbers.
					 */
					err = avCloseStreamFile(stream);
					REPORT_ERR(err);
					track++;
					sprintf(filenameBuild, "track%ld", track);
					filename = filenameBuild;
					open = FALSE;
				}
	
				if (jp.bBtn)
				{
					struct timeval tv = {10, 0};
					avPoint avp = {150, 170, 0};

					/* Print the stream name on screen */
					err = avStartUserFeedbackVA(stream,
					        AV_TAG_UF_STREAMNAME, NULL,
					        AV_TAG_UF_POSITION, (void *)&avp,
					        AV_TAG_UF_DURATION_FIELDS, (void *)300,
					        TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}

				if ((jp.bBtn) && (ABLE(2)))
				{
					/* Get the current position of the stream */
					TagArg gsp[] =
					{
						{AV_TAG_GSP_FIELDS, NULL},
						{AV_TAG_GSP_ABSTIME, NULL},
						{AV_TAG_GSP_BYTES, NULL},
						{AV_TAG_GSP_BLOCKS, NULL},
						{TAG_END, NULL},
					};
					struct timeval *tv;
					avPoint avp = {40, 40, 0};

					DPRT(("@@@ Get position\n"));
					err = avStartUserFeedbackVA(stream,
					      AV_TAG_UF_TIME, NULL,
					      AV_TAG_UF_POSITION, &avp,
					      TAG_END, NULL);
					if (err < 0)
					{
						PrintfSysErr(err);
					}

// 	------------------------------------------------------------------------------------------
//																   avGetStreamPosition tests
		#include "AVtest_GetStreamPosition.c"			
//
// 	------------------------------------------------------------------------------------------
					err = avGetStreamPosition(stream, gsp);
					if (err >= 0)
					{
						kprintf("Fields = %ld\n", gsp[0].ta_Arg);
						tv = (struct timeval *)gsp[1].ta_Arg;
						kprintf("Time  = %ld.%ld secs\n", tv->tv_sec, tv->tv_usec);
						kprintf("Bytes = %ld\n", gsp[2].ta_Arg);
						kprintf("Blocks = %ld\n", gsp[3].ta_Arg);
					}
					else
					{
						PrintfSysErr(err);
					}
				}

				if (jp.cBtn)
				{
					/* Close the stream */
					DPRT(("@@@ close stream\n"));
					if (stream > 0)
					{
						err = avCloseStreamFile(stream);
						REPORT_ERR(err);
					}
					open = FALSE;
					loop = FALSE;
				}
	
				if ((jp.startBtn) && (ABLE(1)))
				{
					/* Toggle between pause and resume, with user feedback */
					if ((pause) || (quick))
					{
						DPRT(("@@@ play normal\n"));
						err = avPlayStream(stream, AV_RATE_NORMAL, userFeedback);
						REPORT_ERR(err);
						quick = FALSE;
						pause = FALSE;
					}
					else
					{
						DPRT(("@@@ pause\n"));
						err = avPlayStream(stream, AV_RATE_PAUSE, userFeedback);
						REPORT_ERR(err);
						pause = TRUE;
					}
				}
			}
			else
			{
				PrintfSysErr(stream);
				loop = FALSE;
			}
			WaitVBL(vbl, 2);
		}

		/* All finished. We no longer need the streamer, so remove it from RAM. */
		DPRT(("@@@ remove streamer\n"));
// 	------------------------------------------------------------------------------------------
//																	  avRemoveStreamer tests
		#include "AVtest_RemoveStreamer.c"
//
// 	------------------------------------------------------------------------------------------
		err = avRemoveStreamer(streamer);
		if (err < 0)
		{
			PrintfSysErr(err);
		}
	}
	else
	{
		PrintfSysErr(streamer);
	}

	if (screenGroupI > 0)
	{
		Close24BitGraphics(&sc, screenGroupI);
	}

	DeleteItem(vbl);
	CloseAVFolio();
	kprintf("See ya\n");
	exit(0);
}
