static char *parse_file( Stream *fp )
{
  static char buff[128];
  char *p, c;

  p = buff;
  buff[0] = 0;

  /* get the first character */
  if( !ReadDiskStream(fp, &c, 1) ) return( buff );

  while( c != '\r' && c != '#' ) {
    *p++ = c;
    if( !ReadDiskStream(fp, &c, 1) )
      c = '\n';
  }

  if( c == '#' )
    while( ReadDiskStream(fp, &c, 1) && c != '\r' )
      ;

  *p = 0; /* null terminate */
  return( buff );
}

void Usage()
{
  kprintf("Usage: either a filename,\n");
  kprintf("or -n <trackNumber>,\n");
  kprintf("-d [16|24] for Depth\n");
  kprintf("-s Screen allocated by this app, not the streamer\n");
  kprintf("-r seRverIP number\n");
  kprintf("-i Iis IP number\n");
  kprintf("-c Client IP number\n");
  kprintf("-p Port number\n");
  kprintf("-t sTreamer name\n");
  kprintf("-f Filename (will use avMPEGStreamer)\n");
  kprintf("-v force Video DMA\n");
  kprintf("-a use cAm instead of RPCs\n");
  kprintf("-b service ID for CAM\n");
  kprintf("-R disable RPCs and CAM - start movie by hand\n");
  kprintf("-U disable User Feedback\n");
  kprintf("\nA button moves on to the next numbered stream (Cable version only\n");
  kprintf("B button prints the stream name on screen and the current\n");
  kprintf("  stream position on the debug console\n");
  kprintf("C button closes the current stream and quits\n");
  kprintf("Left Shift displays debug bars\n");
  kprintf("Stop button stops the stream and closes it\n");
  kprintf("Pause button pauses/resumes the stream\n");
  kprintf("Up/Down changes volume\n");
  kprintf("Left/Right mutes/unmutes\n");
  CloseAVFolio();
  exit(0);
}

void PrintCOS(CAMOpenServiceReply *cosrep)
{
	kprintf("  num_CAMOSI        = %d\n",cosrep->num_CAMOSI);
	kprintf("  subServiceCount   = %d\n",cosrep->subServiceCount);
	kprintf("  currentServerTime = %d\n",cosrep->currentServerTime);
	kprintf("  serverTimeZone    = %d\n",cosrep->serverTimeZone);
	kprintf("  videoStartTime    = %d\n",cosrep->videoStartTime);
	kprintf("\n  CAMOSI [0x%x]\n",cosrep->CAMOSI);
	kprintf("  serviceAddress        = 0x%x:0x%x\n",
		cosrep->CAMOSI->serviceAddress.ipAddress,
		cosrep->CAMOSI->serviceAddress.port);
	kprintf("  protocol              = %d\n",cosrep->CAMOSI->protocol);
	kprintf("  serviceProgramID      = %d\n",cosrep->CAMOSI->serviceProgramID);
	kprintf("  serviceProgramVersion = %d\n",cosrep->CAMOSI->serviceProgramVersion);
	kprintf("\n  serviceInfo [0x%x]\n",cosrep->CAMOSI->serviceInfo);
	kprintf("  serviceID      = %d\n",cosrep->CAMOSI->serviceInfo->serviceID);
	kprintf("  serviceType    = %d\n",cosrep->CAMOSI->serviceInfo->serviceType);
	kprintf("  serviceVersion = %d\n",cosrep->CAMOSI->serviceInfo->serviceVersion);
	kprintf("  serviceName    = %s\n",cosrep->CAMOSI->serviceInfo->serviceName);
	kprintf("  serviceInfoFlg = 0x%x\n",cosrep->CAMOSI->serviceInfo->serviceInfoFlg);
}

CAMOpenServiceInfo *GetCAMOSI(CAMOpenServiceReply *cosrep)
{
	return(cosrep->CAMOSI);
}

#if 0
uint32 inet_addr( char *cp )
{
  register uint32 val, base, n;
  register char c;
  uint32 parts[4], *pp = parts;
  
//  SPRT(("inet_addr: %s\n",cp));
  
 again:
  /*
   * Collect number up to ``.''.
   * Values are specified as for C:
   * 0x=hex, 0=octal, other=decimal.
   */
  val = 0; base = 10;
  if (*cp == '0') {
    if (*++cp == 'x' || *cp == 'X')
      base = 16, cp++;
    else
      base = 8;
  }
  while ( (c = *cp) != 0 ) {
    if (isdigit(c)) {
      val = (val * base) + (uint32)(c - '0');
      cp++;
      continue;
    }
    if (base == 16 && isxdigit(c)) {
      val = (val << 4) + (uint32)(c + 10 - (islower(c) ? 'a' : 'A'));
      cp++;
      continue;
    }
    break;
  }
  if (*cp == '.') {
    /*
     * Internet format:
     *	a.b.c.d
     *	a.b.c	(with c treated as 16-bits)
     *	a.b	(with b treated as 24 bits)
     */
    if (pp >= parts + 4)
      return ( 0 );
    *pp++ = val, cp++;
    goto again;
  }
  /*
   * Check for trailing characters.
   */
  if (*cp && !isspace(*cp))
    return ( 0 );
  *pp++ = val;
  /*
   * Concoct the address according to
   * the number of parts specified.
   */
  n = pp - parts;
  switch (n) {
    
  case 1:				/* a -- 32 bits */
    val = parts[0];
    break;
    
  case 2:				/* a.b -- 8.24 bits */
    val = (parts[0] << 24) | (parts[1] & 0xffffff);
    break;
    
  case 3:				/* a.b.c -- 8.8.16 bits */
    val = (parts[0] << 24) | ((parts[1] & 0xff) << 16) |
      (parts[2] & 0xffff);
    break;
    
  case 4:				/* a.b.c.d -- 8.8.8.8 bits */
    val = (parts[0] << 24) | ((parts[1] & 0xff) << 16) |
      ((parts[2] & 0xff) << 8) | (parts[3] & 0xff);
    break;
    
  default:
    return ( 0 );
  }
//  val = htonl(val);
//  SPRT(("inet_addr: val = 0x%lx\n",val));
  return (val);
}

#endif
