/**
|||	AUTODOC PUBLIC sample/ta_delay
|||	ta_delay  - alpha autodoc header..summary to be added later
|||
|||	  Synopsis
|||
|||	    ta_delay [arg1] [arg2]...
|||
|||	  Description
|||
|||	    ALPHA DESCRIPTION -- WILL BE FILLED LATER
|||
|||	  Arguments
|||
|||	    arg1			arg1 description
|||
|||	    arg2			arg2 description
|||
|||	  Return Value
|||
|||	    This demo exits 0 for normal execution, and -1 in case of error
|||
|||
|||	  Implementation
|||
|||	    This sample code is prepared for compilation with Portfolio 2.0
|||
|||	  Caveats
|||
|||	    Alpha autodoc header -- more description to be added later
|||
|||	  Associated Files
|||
|||	    Alpha autodoc header -- more description to be added later
|||
**/
/* $Id: ta_delay.c,v 1.5 1994/08/30 22:36:38 jbyrd Exp $ */
/***************************************************************
**
** Test delay1tap.dsp
** By:  Phil Burk
**
** Copyright (c) 1994, 3DO Company.
** This program is proprietary and confidential.
**
***************************************************************/

#include "types.h"
#include "filefunctions.h"
#include "debug.h"
#include "operror.h"
#include "stdio.h"
#include "stdlib.h"

/* Include this when using the Audio Folio */
#include "audio.h"

#include <handy_tools.h>            /* Choose() */

#define	PRT(x)	{ printf x; }
#define	ERR(x)	PRT(x)
#define	DBUG(x)	PRT(x)

#define INSNAME "sampler.dsp"

int32 PlayFreqNote ( Item Instrument, int32 Freq, int32 Duration );

/* Macro to simplify error checking. */
#define CHECKRESULT(val,name) \
	if (val < 0) \
	{ \
		Result = val; \
		ERR(("Failure in %s: $%x\n", name, val)); \
		PrintfSysErr(Result); \
		goto cleanup; \
	}


int32 PlayPitchNote ( Item Instrument, int32 Note, int32 Velocity, int32 Duration);

#define NUM_MIXER_CHANNELS (4)
typedef struct DelayContext
{
	Item dlc_DelayIns;
	Item dlc_DelayLine;
	Item dlc_Tap0Send;
	Item dlc_Tap1Send;
	Item dlc_Tap2Send;
	Item dlc_OriginalSend;
	Item dlc_Tap0Mix;
	Item dlc_Tap1Mix;
	Item dlc_Tap2Mix;
	Item dlc_OriginalMix;
} DelayContext;

int32 Init1TapDelay( DelayContext *dlc, int32 DelaySize, int32 DelayFrames);
int32 Term1TapDelay( DelayContext *dlc );

#define DELAY_FRAMES   (100000)

/********************************************************************/
int main(int argc, char *argv[])
{
	Item SamplerIns = 0;
	Item SampleItem = 0;
	Item Attachment = 0;
	Item OutputIns = 0;
	DelayContext DelayCon;
	int32 Duration, i;
	int32 Result = -1;
	char *SampleName;
	int32 DelayFrames;


	PRT(("%s <samplefile>\n", argv[0]));

	SampleName = (argc > 1) ? argv[1] : "$samples/bass.aiff";
	Duration = (argc > 2) ? atoi(argv[2]) : 240 ;
	PRT(("Duration = %d\n", Duration));
	DelayFrames = (argc > 3) ? atoi(argv[3]) : DELAY_FRAMES ;
	PRT(("DelayFrames = %d\n", DelayFrames));

/* Initialize audio, return if error. */
	if (OpenAudioFolio())
	{
		ERR(("Audio Folio could not be opened!\n"));
		return(-1);
	}

TraceAudio(0);
	Result = Init1TapDelay( &DelayCon, DelayFrames*sizeof(int16), DelayFrames - 100);
	CHECKRESULT(Result,"Init1TapDelay");

	OutputIns = LoadInstrument("directout.dsp", 0, 0);
	CHECKRESULT(OutputIns,"LoadInstrument");

/* Load Sampler instrument */
	SamplerIns = LoadInstrument(INSNAME, 0, 100);
	CHECKRESULT(SamplerIns,"LoadInstrument");

/* Load digital audio Sample from disk. */
	SampleItem = LoadSample(SampleName);  /* from command line? */
	CHECKRESULT(SampleItem,"LoadSample");


/* Look at sample information. */
	DebugSample(SampleItem);

/* Connect Sampler to Delay */
	Result = ConnectInstruments (SamplerIns, "Output",
		DelayCon.dlc_DelayIns, "Input");
	CHECKRESULT(Result,"ConnectInstruments");

/* Connect Delay to Output */
	Result = ConnectInstruments (DelayCon.dlc_DelayIns, "Output",
		OutputIns, "InputLeft");
	CHECKRESULT(Result,"ConnectInstruments");
	Result = ConnectInstruments (DelayCon.dlc_DelayIns, "Output",
		OutputIns, "InputRight");
	CHECKRESULT(Result,"ConnectInstruments");

/* Attach the sample to the instrument. */
	Attachment = AttachSample(SamplerIns, SampleItem, 0);
	CHECKRESULT(Attachment,"AttachSample to SamplerIns");

/* Output must be started */
	Result = StartInstrument( OutputIns, NULL );
	CHECKRESULT(Result,"StartInstrument");

/* Start Delay */
	Result = StartInstrument( DelayCon.dlc_DelayIns, NULL );
	CHECKRESULT(Result,"StartInstrument");

/* Play several notes using a conveniance routine. */
	PlayPitchNote( SamplerIns, 40, 100, Duration );
	PlayPitchNote( SamplerIns, 44,  64, Duration );
	PlayPitchNote( SamplerIns, 47,  64, Duration );
	PlayPitchNote( SamplerIns, 52,  64, Duration );
	PlayPitchNote( SamplerIns, 59, 100, Duration );
	PlayPitchNote( SamplerIns, 64,  64, Duration*2 );
	for( i=0; i<16; i++)
	{
		PlayPitchNote( SamplerIns, Choose(24)+48,  64, Duration>>2 );
	}


/* Let echoes die down. */
	SleepAudioTicks(2000);

cleanup:

	Term1TapDelay( &DelayCon );
	DetachSample( Attachment );
	UnloadInstrument( SamplerIns );
	UnloadInstrument( OutputIns );
	UnloadSample( SampleItem );

PRT(("All Done---------------------------------\n"));
	CloseAudioFolio();
	return((int) Result);
}

/********************************************************************/
/***** Play a note based on MIDI pitch. *****************************/
/********************************************************************/

/* Define Tags for StartInstrument */
TagArg SamplerTags[] =
	{
		{ AF_TAG_VELOCITY, 0},
		{ AF_TAG_PITCH, 0},
        { TAG_END, 0 }
    };

int32 PlayPitchNote ( Item Instrument, int32 Note, int32 Velocity, int32 Duration)
{
	int32 Result;

	SamplerTags[0].ta_Arg = (void *) Velocity;
	SamplerTags[1].ta_Arg = (void *) Note;
	Result = StartInstrument( Instrument, &SamplerTags[0] );
	Result = SleepAudioTicks( Duration>>1 );

	ReleaseInstrument( Instrument, NULL);
	Result = SleepAudioTicks( Duration>>1 );
	return 0;
}

/*********************************************************************/
int32 Init1TapDelay( DelayContext *dlc, int32 DelaySize, int32 DelayFrames)
{
	int32 Result=0;
	Item  Attachment;
	TagArg Tags[2];

/*
** Create a delay line.  This must be allocated by the AudioFolio
** because the memory is written to by hardware.  A delay line
** is just a sample with a special write permission.
** 1=channel, TR loop
*/
#define NUM_CHANNELS (1)
#define IF_LOOP      (TRUE)
	dlc->dlc_DelayLine = CreateDelayLine( DelaySize, NUM_CHANNELS, IF_LOOP );
	CHECKRESULT(dlc->dlc_DelayLine,"CreateDelayLine");

/*
** Load a delay instrument with 1 tap
** Other delays are available and custom delays can be constructed.
*/
	dlc->dlc_DelayIns = LoadInstrument("delay1tap.dsp", 0, 100);
	CHECKRESULT(dlc->dlc_DelayIns,"LoadInstrument");

/* Attach the delay line to the delay instrument output. */
	Attachment = AttachSample( dlc->dlc_DelayIns, dlc->dlc_DelayLine, "OutFIFO" );
	CHECKRESULT(Attachment,"AttachSample to dlc->dlc_DelayIns OutFIFO");

/* Set STARTAT value to determine delay time. */
	Tags[0].ta_Tag = AF_TAG_START_AT;
	Tags[0].ta_Arg = (void *) DelayFrames;
	Tags[1].ta_Tag = TAG_END;
	Result = SetAudioItemInfo( Attachment, Tags );
	CHECKRESULT(Result,"SetAudioItemInfo: START_AT");

/* Attach the delay line to the delay instrument tap. */
	Attachment = AttachSample( dlc->dlc_DelayIns, dlc->dlc_DelayLine, "Tap0" );
	CHECKRESULT(Attachment,"AttachSample to dlc->dlc_DelayIns Tap0");

/* Attach the various gain knobs. */
	dlc->dlc_Tap0Send = GrabKnob( dlc->dlc_DelayIns, "Tap0Send" );
	CHECKRESULT(dlc->dlc_Tap0Send,"GrabKnob");
	dlc->dlc_Tap0Mix = GrabKnob( dlc->dlc_DelayIns, "Tap0Mix" );
	CHECKRESULT(dlc->dlc_Tap0Mix,"GrabKnob");
	dlc->dlc_OriginalSend = GrabKnob( dlc->dlc_DelayIns, "OriginalSend" );
	CHECKRESULT(dlc->dlc_OriginalSend,"GrabKnob");
	dlc->dlc_OriginalMix = GrabKnob( dlc->dlc_DelayIns, "OriginalMix" );
	CHECKRESULT(dlc->dlc_OriginalMix,"GrabKnob");

/* Set Mixer Levels */
	TweakKnob ( dlc->dlc_Tap0Send, -0x4000 );
	TweakKnob ( dlc->dlc_OriginalSend, 0x4000 );
	TweakKnob ( dlc->dlc_Tap0Mix, 0x4000 );
	TweakKnob ( dlc->dlc_OriginalMix, 0x4000 );
	return Result;

cleanup:
	Term1TapDelay( dlc );
	return Result;
}


int32 Term1TapDelay( DelayContext *dlc )
{
	UnloadInstrument( dlc->dlc_DelayIns );
	UnloadSample( dlc->dlc_DelayLine );
	ReleaseKnob( dlc->dlc_OriginalSend );
	ReleaseKnob( dlc->dlc_OriginalMix );
	ReleaseKnob( dlc->dlc_Tap0Send );
	ReleaseKnob( dlc->dlc_Tap0Mix );
	return 0;
}
