/*****

$Id: Walker.c,v 1.10 1994/05/05 23:23:18 martin Exp $

$Log: Walker.c,v $
 * Revision 1.10  1994/05/05  23:23:18  martin
 * now includes only necessary includes
 * uses CreateIOReq() instead of CreateItem()
 * uses printf() instead of kprintf()
 *
 * Revision 1.9  1994/02/18  17:42:53  limes
 * include "operror.h" to get PrintError macro
 *
 * Revision 1.8  1994/02/18  01:54:55  limes
 * enhanced error reporting
 *
 * Revision 1.7  1993/03/16  06:36:37  dplatt
 * Functional Freeze release
 *
 * Revision 1.6  1993/02/11  19:39:37  dplatt
 * Developer-release and new-kernel changes
 *
 * Revision 1.5  1992/12/08  05:59:52  dplatt
 * Magenta changes
 *
 * Revision 1.4  1992/10/24  00:40:56  dplatt
 * Bluebird changes and bug fixes
 *
 * Revision 1.3  1992/10/20  06:02:33  dplatt
 * Blue changes
 *
 * Revision 1.2  1992/10/01  23:36:21  dplatt
 * Switch to int32/uint32
 *
 * Revision 1.1  1992/09/11  00:42:31  dplatt
 * Initial revision
 *

 *****/

/*
  Copyright New Technologies Group, 1991.
  All Rights Reserved Worldwide.de_
  Company confidential and proprietary.
  Contains unpublished technical data.
*/

/*
  Walker.c - Sample code to scan the file system.
*/

#include "types.h"
#include "io.h"
#include "stdio.h"
#include "operror.h"
#include "discdata.h"

#include "filesystem.h"
#include "directory.h"
#include "directoryfunctions.h"
#include "filefunctions.h"


/*
   Sample code to print entries of an Opera directory hierarchy.
*/

void TraverseDirectory(Item openFileItem)
{
  Directory *dir;
  DirectoryEntry de;
  Item subDirItem;
  Item ioReqItem;
  int32 err, entry;
  uint32 block, done;
  block = 0;
  done = 0;

  ioReqItem = CreateIOReq( NULL, 0, openFileItem, 0 );
  if (ioReqItem < 0) {
    printf("Couldn't create IOReq for open-file item %d\n", openFileItem);
    return;
  }
  dir = OpenDirectoryItem(openFileItem);
  if (!dir) {
    printf("Could not open a directory for item %d\n", openFileItem);
    return;
  }
  entry = 1;
  while (1) {
    err = ReadDirectory(dir, &de);
    if (err < 0) break;  /* Testing the error code would be good... */
    printf("Entry #%d:\n", entry);
    printf("  file '%s', type 0x%lx, ID 0x%lx",
	    de.de_FileName, de.de_Type,
	    de.de_UniqueIdentifier);
    printf(", flags 0x%lx\n", de.de_Flags);
    printf("  %d bytes, %d blocks of %d bytes each\n",
	    de.de_ByteCount,
	    de.de_BlockCount, de.de_BlockSize);
    printf("  %d avatar(s)", de.de_AvatarCount);
#ifdef PRINTAVATARS
    for (avatarIndex = 1;
	 avatarIndex <= de.de_LastAvatarIndex;
	 avatarIndex++) {
      printf(", %d", de.de_AvatarList[avatarIndex]);
    }
#endif
    printf("\n\n");
    if (de.de_Flags & FILE_IS_DIRECTORY) {
      subDirItem = OpenDiskFileInDir(openFileItem, de.de_FileName);
      if (subDirItem >= 0) {
	printf("********** RECURSE **********\n\n");
	TraverseDirectory(subDirItem);
	(void) CloseDiskFile(subDirItem);
	printf("******* END RECURSION *******\n\n");
      } else {
	printf("***  RECURSION FAILED  ***\n\n");
      }
    }	
    entry ++;
  }
  CloseDirectory(dir);
}

int32 Walk (char *path)
{
  Item startItem;
  printf("\nRecursive directory scan from %s\n\n", path);
  startItem = OpenDiskFile(path);
  if (startItem < 0) {
    PrintError(0,"open",path,startItem);
    return -999;
  }
  TraverseDirectory(startItem);
  CloseDiskFile(startItem);
  printf("End of %s has been reached\n\n", path);
  return 0;
}

int main (int32 argc, char **argv)
{
  int i;
#ifdef DEBUG
  printf("Walker here, argc = %d\n", argc);
  for (i = 1; i < argc; i ++) {
    printf("%s ", argv[i]);
  }
  printf("\n");
#endif
  if (argc < 2) {
    Walk(".");
  } else {
    for (i = 1; i < argc; i ++) {
      Walk(argv[i]);
    }
  }
  return 0;
}







