/**
|||	AUTODOC PUBLIC sample/compressexample
|||	compressexample  - alpha autodoc header..summary to be added later
|||
|||	  Synopsis
|||
|||	    compressexample [arg1] [arg2]...
|||
|||	  Description
|||
|||	    ALPHA DESCRIPTION -- WILL BE FILLED LATER
|||
|||	  Arguments
|||
|||	    arg1			arg1 description
|||
|||	    arg2			arg2 description
|||
|||	  Return Value
|||
|||	    This demo exits 0 for normal execution, and -1 in case of error
|||
|||
|||	  Implementation
|||
|||	    This sample code is prepared for compilation with Portfolio 2.0
|||
|||	  Caveats
|||
|||	    Alpha autodoc header -- more description to be added later
|||
|||	  Associated Files
|||
|||	    Alpha autodoc header -- more description to be added later
|||
**/

/* $Id: compressexample.c,v 1.2 1994/09/02 18:21:22 benn Exp $ */

/* This is a simple program demonstrating how to use the compression routines
 * supplied by compression folio. If you run this program, it will
 * load itself into a memory buffer, compress the data, decompress
 * it, and will compare the original data with the decompressed data to
 * make sure the compression process worked successfully.
 */

#include "types.h"
#include "filestream.h"
#include "filestreamfunctions.h"
#include "stdio.h"
#include "mem.h"
#include "compression.h"


/*****************************************************************************/


int main(int argc, char **argv)
{
Stream       *stream;
Err           err;
bool          same;
uint32        i;
int32         fileSize;
uint32       *originalData;
uint32       *compressedData;
uint32       *finalData;
int32         numFinalWords;
int32         numCompWords;

    err = OpenCompressionFolio();
    if (err >= 0)
    {
        stream = OpenDiskStream(argv[0],0);
        if (stream)
        {
            fileSize       = stream->st_FileLength & 0xfffffffc;
            originalData   = (uint32 *)malloc(fileSize);
            compressedData = (uint32 *)malloc(fileSize);
            finalData      = (uint32 *)malloc(fileSize);

            if (originalData && compressedData && finalData)
            {
                if (ReadDiskStream(stream,(char *)originalData,fileSize) == fileSize)
                {
                    err = Compress(originalData, fileSize / sizeof(uint32),
                                   compressedData, fileSize / sizeof(uint32));
                    if (err >= 0)
                    {
                        numCompWords = err;
                        err = Decompress(compressedData, numCompWords,
                                         finalData, fileSize / sizeof(uint32));
                        if (err >= 0)
                        {
                            numFinalWords = err;
                            printf("Original data size    : %d\n",fileSize / sizeof(uint32));
                            printf("Compressed data size  : %d\n",numCompWords);
                            printf("Uncompressed data size: %d\n",numFinalWords);

                            same = TRUE;
                            for (i = 0; i < fileSize / sizeof(uint32); i++)
                            {
                                if (originalData[i] != finalData[i])
                                {
                                    same = FALSE;
                                    break;
                                }
                            }

                            if (same)
                            {
                                printf("Uncompressed data matched original\n");
                            }
                            else
                            {
                                printf("Uncompressed data differed with original!\n");
                                for (i = 0; i < 10; i++)
                                {
                                    printf("orig $%08x, final $%08x, comp $%08x\n",
                                           originalData[i],
                                           finalData[i],
                                           compressedData[i]);
                                }
                            }
                        }
                        else
                        {
                            PrintError(NULL,"decompress data",NULL,err);
                        }
                    }
                    else
                    {
                        PrintError(NULL,"compress data",NULL,err);
                    }
                }
                else
                {
                    printf("Could not read whole file\n");
                }
            }
            else
            {
                printf("Could not allocate memory buffers\n");
            }

            free(originalData);
            free(compressedData);
            free(finalData);

            CloseDiskStream(stream);
        }
        else
        {
            printf("Could not open '%s' as an input file\n",argv[0]);
        }
        CloseCompressionFolio();
    }
    else
    {
        PrintError(NULL,"open the compression folio",NULL,err);
    }

    return (0);
}
